# -*- coding: utf-8 -*-
"""Material used to generate and execute bindings generated by the PyXB package.
"""

# Automatically include all the stuff that's necessary for basic
# running.  There are order dependencies here.
from . import basis
from . import datatypes
from . import facets
from . import content

# Do not include the stuff that's required only for code generation
# noimport generate



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:tns="urn:trac-0069" targetNamespace="urn:trac-0069">
  <xs:element name="MetadataDocument" type="tns:MetadataType"/>
  <xs:complexType name="MetadataType">
    <xs:sequence maxOccurs="1" minOccurs="1">
      <xs:element name="template" type="xs:string"/>
      <xs:element name="timespan" maxOccurs="unbounded" minOccurs="0">
        <xs:complexType>
          <xs:sequence>
            <xs:element name="field" maxOccurs="unbounded" minOccurs="0">
              <xs:complexType>
                <xs:sequence>
                  <xs:element name="name" type="xs:string"/>
                  <xs:element name="value" minOccurs="0" maxOccurs="unbounded">
                    <xs:complexType>
                      <xs:simpleContent>
                        <xs:extension base="xs:string">
                          <xs:attribute name="lang" type="xs:language"/>
                          <xs:attribute name="user" type="xs:string"/>
                          <xs:attribute name="timestamp" type="xs:dateTime"/>
                        </xs:extension>
                      </xs:simpleContent>
                    </xs:complexType>
                  </xs:element>
                </xs:sequence>
                <xs:attribute name="track" type="xs:string"/>
              </xs:complexType>
            </xs:element>
          </xs:sequence>
          <xs:attribute name="start" type="xs:string"/>
          <xs:attribute name="end" type="xs:string"/>
        </xs:complexType>
      </xs:element>
    </xs:sequence>
  </xs:complexType>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest
import collections

# Pretend whoever created the schema was helpful and had normalized it
metadatadoc_type = MetadataDocument.typeDefinition()
timespan_element = metadatadoc_type._ElementMap['timespan'].elementBinding()
timespan_type = timespan_element.typeDefinition()
field_element = timespan_type._ElementMap['field'].elementBinding()
field_type = field_element.typeDefinition()
value_element = field_type._ElementMap['value'].elementBinding()
value_type = value_element.typeDefinition()

v_bind = pyxb.BIND('foo', lang='ENG')

class TestTrac_0069 (unittest.TestCase):
    def testMetaConstructor (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        newdoc.timespan.append(pyxb.BIND(start='-INF', end='+INF'))
        timespan = newdoc.timespan[0]
        self.assertTrue(isinstance(timespan, timespan_type))
        timespan.field.append(pyxb.BIND('name', pyxb.BIND('fv0'), pyxb.BIND('fv1')))
        field = timespan.field[0]
        self.assertTrue(isinstance(field, field_type))
        field.value_.append('fv2')
        fv2 = field.value_[2]
        self.assertTrue(isinstance(fv2, value_type))
        self.assertEqual('fv2', fv2.value())
        newdoc.validateBinding()
        xmlt = six.u('<ns1:MetadataDocument xmlns:ns1="urn:trac-0069"><template>anewtemplate</template><timespan end="+INF" start="-INF"><field><name>name</name><value>fv0</value><value>fv1</value><value>fv2</value></field></timespan></ns1:MetadataDocument>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(newdoc.toxml("utf-8", root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import dict
from pyxb.utils.six.moves.urllib.request import urlopen
import sys
from pyxb.utils import domutils

word = 'xml'
if 1 < len(sys.argv):
    word = sys.argv[1]

# Create a REST-style query to retrieve the information about this dictionary.
uri = 'http://services.aonaware.com/DictService/DictService.asmx/Define?word=%s' % (word,)
rxml = urlopen(uri).read()
resp = dict.CreateFromDOM(domutils.StringToDOM(rxml))

print('Definitions of %s:' % (resp.Word,))
for definition in resp.Definitions.Definition:
    print('From %s (%s):' % (definition.Dictionary.Name, definition.Dictionary.Id))
    print(definition.WordDefinition)
    print()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="tNumber">
    <xs:simpleContent>
      <xs:extension base="xs:double">
        <xs:attribute name="scale" type="xs:int"/>
      </xs:extension>
    </xs:simpleContent>
  </xs:complexType>
  <xs:element name="number" type="tNumber"/>
  <xs:element name="numbers">
    <xs:complexType>
      <xs:sequence>
        <xs:element ref="number"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0232 (unittest.TestCase):
    def testValid (self):
        instance = CreateFromDocument('<numbers><number scale="2">1.5</number></numbers>')
        self.assertEqual(1.5, instance.number.value())
        self.assertEqual(2, instance.number.scale)

    def testInvalidElement (self):
        with self.assertRaises(pyxb.SimpleTypeValueError) as cm:
            instance = CreateFromDocument('''<numbers>
<number>1x5</number>
</numbers>''')
        e = cm.exception
        # NB: Location is the start tag of the containing element
        if e.location is not None:
            self.assertEqual(2, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)

    def testInvalidAttribute (self):
        with self.assertRaises(pyxb.SimpleTypeValueError) as cm:
            instance = CreateFromDocument('''<numbers><number scale="c">1.5</number></numbers>''')
        e = cm.exception
        # NB: Location is the start tag of the containing element
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(9, e.location.columnNumber)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import unittest
import pyxb.binding.datatypes as xsd

class Test_token (unittest.TestCase):
    Cases = [ ('Internal spaces are ok', None),
                  ("with\nnewline", 'with newline'),
                  ("with\rreturn", 'with return'),
                  ("with\ttab", 'with tab'),
                  ("\n\nleading newline", 'leading newline'),
                  ("trailing newline\n\n", 'trailing newline'),
                  (' LeadingSpace', 'LeadingSpace'),
                  ('TrailingSpace ', 'TrailingSpace'),
                  ('Internal  Multiple Spaces', 'Internal Multiple Spaces'),
                  ]

    def testValid (self):
        for (lexical, value) in self.Cases:
            if value is None:
                value = lexical
            self.assertEqual(value, xsd.token(value))
            self.assertEqual(value, xsd.token(lexical, _from_xml=True))

    def testInvalid (self):
        for (lexical, value) in self.Cases:
            if value is not None:
                self.assertRaises(pyxb.SimpleTypeValueError, xsd.token, lexical)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six
from pyxb.namespace.builtin import XMLSchema_instance as XSI

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:complexType name="BaseT" abstract="true"/>
<xs:complexType name="String">
  <xs:complexContent>
    <xs:extension base="BaseT">
      <xs:sequence>
        <xs:element name="child" type="xs:string"/>
      </xs:sequence>
    </xs:extension>
  </xs:complexContent>
</xs:complexType>
<xs:complexType name="Integer">
  <xs:complexContent>
    <xs:extension base="BaseT">
      <xs:sequence>
        <xs:element name="child" type="xs:integer"/>
      </xs:sequence>
    </xs:extension>
  </xs:complexContent>
</xs:complexType>
<xs:complexType name="Foreign">
  <xs:sequence>
    <xs:element name="child" type="xs:string"/>
  </xs:sequence>
</xs:complexType>
<xs:element name="base" type="BaseT"/>
<xs:element name="string" type="String"/>
<xs:element name="integer" type="Integer"/>
<xs:element name="wildcard">
  <xs:complexType>
    <xs:sequence>
      <xs:any/>
    </xs:sequence>
  </xs:complexType>
</xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0060 (unittest.TestCase):
    def setUp (self):
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel
        XSI.ProcessTypeAttribute(XSI.PT_strict)

    AString = 'hi'
    NotAnInteger = 'not an integer'
    AnInteger = 34
    BaseUntyped = '<base><child>%s</child></base>' % (NotAnInteger,)
    BaseString = '<base xsi:type="String" xmlns:xsi="%s"><child>%s</child></base>' % (XSI.uri(), NotAnInteger)
    BaseInteger = '<base xsi:type="Integer" xmlns:xsi="%s"><child>%d</child></base>' % (XSI.uri(), AnInteger)
    BaseForeign = '<base xsi:type="Foreign" xmlns:xsi="%s"><child>%s</child></base>' % (XSI.uri(), NotAnInteger)
    BaseUnknown = '<base xsi:type="Unknown" xmlns:xsi="%s"><child>%s</child></base>' % (XSI.uri(), NotAnInteger)
    CorrectString = '<string><child>%s</child></string>' % (AString,)
    CorrectInteger = '<integer><child>%d</child></integer>' % (AnInteger,)
    BadInteger = '<integer><child>%s</child></integer>' % (NotAnInteger,)
    ConflictString = '<integer xsi:type="String" xmlns:xsi="%s"><child>%s</child></integer>' % (XSI.uri(), NotAnInteger)
    ConflictStringInteger = '<integer xsi:type="String" xmlns:xsi="%s"><child>%d</child></integer>' % (XSI.uri(), AnInteger)
    UntypedIntegerElement = '<element><child>%d</child></element>' % (AnInteger,)
    TypedElement = '<element xsi:type="Integer" xmlns:xsi="%s"><child>%d</child></element>' % (XSI.uri(), AnInteger)
    # There is no type Float in the schema
    BogusTypedElement = '<element xsi:type="Float" xmlns:xsi="%s"><child>%d</child></element>' % (XSI.uri(), AnInteger)
    BogusTypedInteger = '<integer xsi:type="Float" xmlns:xsi="%s"><child>%d</child></integer>' % (XSI.uri(), AnInteger)

    def makeWC (self, body):
        return '<wildcard>%s</wildcard>' % (body,)

    def testWildcardString (self):
        xmls = '<wildcard>%s</wildcard>' % (self.CorrectString,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual(self.AString, instance.child)

    def testWildcardInteger (self):
        xmls = '<wildcard>%s</wildcard>' % (self.CorrectInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)

    def testWildcardIntegerBad (self):
        xmls = '<wildcard>%s</wildcard>' % (self.BadInteger,)
        self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDocument, xmls)

    def testWildcardUntyped (self):
        # Inhibit warning about conversion; we get a DOM node out of this.
        self.__basis_log.level = logging.ERROR
        xmls = '<wildcard>%s</wildcard>' % (self.UntypedIntegerElement,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertFalse(isinstance(instance, pyxb.binding.basis._TypeBinding_mixin))

    def testWildcardTyped (self):
        xmls = '<wildcard>%s</wildcard>' % (self.TypedElement,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))

    def testBase (self):
        xmls = self.makeWC(self.BaseString)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual(self.NotAnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual(self.NotAnInteger, instance.child)

        xmls = '<wildcard>%s</wildcard>' % (self.BaseInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)

    def testStrict (self):
        self.assertEqual(XSI.ProcessTypeAttribute(), XSI.PT_strict)
        xmls = '<wildcard>%s</wildcard>' % (self.ConflictString,)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDOM, dom)

        xmls = '<wildcard>%s</wildcard>' % (self.ConflictStringInteger,)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDOM, dom)

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedElement,)
        # This only raises with saxer, which treats the bogus xsi:type as
        # a content failure.  With saxdom and minidom, that error gets
        # lost by the code that thinks it's ok to pass a DOM node where
        # no binding could be created.  This should be reconciled.
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedInteger,)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)

        xmls = self.makeWC(self.BaseUntyped)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseForeign)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseUnknown)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDOM, dom)

    def testLax (self):
        # Lax won't convert DOM nodes
        self.__basis_log.level = logging.ERROR
        self.assertEqual(XSI.ProcessTypeAttribute(), XSI.PT_strict)
        XSI.ProcessTypeAttribute(XSI.PT_lax)
        xmls = '<wildcard>%s</wildcard>' % (self.ConflictString,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual(self.NotAnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual(self.NotAnInteger, instance.child)

        xmls = '<wildcard>%s</wildcard>' % (self.ConflictStringInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual('%d' % (self.AnInteger,), instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, String))
        self.assertEqual('%d' % (self.AnInteger,), instance.child)

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedElement,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertFalse(isinstance(instance, pyxb.binding.basis._TypeBinding_mixin))
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertFalse(isinstance(instance, pyxb.binding.basis._TypeBinding_mixin))

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)

        xmls = self.makeWC(self.BaseUntyped)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseForeign)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Foreign))
        self.assertEqual(self.NotAnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Foreign))
        self.assertEqual(self.NotAnInteger, instance.child)

        xmls = self.makeWC(self.BaseUnknown)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)


    def testSkip (self):
        # Skip won't convert DOM nodes
        self.__basis_log.level = logging.ERROR
        self.assertEqual(XSI.ProcessTypeAttribute(), XSI.PT_strict)
        XSI.ProcessTypeAttribute(XSI.PT_skip)

        # skip uses element name to force to integer, which content doesn't match
        xmls = '<wildcard>%s</wildcard>' % (self.ConflictString,)
        self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDOM, dom)

        xmls = '<wildcard>%s</wildcard>' % (self.ConflictStringInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertEqual(self.AnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertEqual(self.AnInteger, instance.child)

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedElement,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertFalse(isinstance(instance, pyxb.binding.basis._TypeBinding_mixin))
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertFalse(isinstance(instance, pyxb.binding.basis._TypeBinding_mixin))

        xmls = '<wildcard>%s</wildcard>' % (self.BogusTypedInteger,)
        wc = CreateFromDocument(xmls)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        wc = CreateFromDOM(dom)
        instance = wc.wildcardElements()[0]
        self.assertTrue(isinstance(instance, Integer))
        self.assertTrue(isinstance(instance.child, six.long_type))
        self.assertEqual(self.AnInteger, instance.child)

        xmls = self.makeWC(self.BaseUntyped)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseString)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseInteger)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseForeign)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

        xmls = self.makeWC(self.BaseUnknown)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDocument, xmls)
        dom = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, dom)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

from pyxb.exceptions_ import *

import unittest

class TestIssue0043 (unittest.TestCase):
    def testFloat (self):
        self.assertEqual('INF', xs.float('Infinity').xsdLiteral())
        self.assertEqual('-INF', xs.float('-Infinity').xsdLiteral())
        self.assertEqual('NaN', xs.float('nan').xsdLiteral())

    def testDouble (self):
        self.assertEqual('INF', xs.double('Infinity').xsdLiteral())
        self.assertEqual('-INF', xs.double('-Infinity').xsdLiteral())
        self.assertEqual('NaN', xs.double('nan').xsdLiteral())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import common4app
import common

class Test (unittest.TestCase):
    def testExtended (self):
        x = common4app.extended('hi', 'there')
        self.assertEqual(x.elt, 'hi')
        self.assertEqual(x.extElt, 'there')
        self.assertTrue(issubclass(common4app.extended.typeDefinition(), common.base.typeDefinition()))

    def testNamespaceInfo (self):
        ns = common.Namespace
        ns.validateComponentModel()
        self.assertEqual(0, len(ns.moduleRecords()))

if '__main__' == __name__:
    unittest.main()



from __future__ import print_function
import po3

order = po3.CreateFromDocument(open('po3.xml').read())

print('%s is sending %s %d thing(s):' % (order.billTo.name, order.shipTo.name, len(order.items.item)))
for item in order.items.item:
    print('  Quantity %d of %s at $%s' % (item.quantity, item.productName, item.USPrice))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)

import sys
import pyxb
import unittest

class TestTrac0132 (unittest.TestCase):
    message = 'bad character \u2620'
    def testDecode (self):
        e = pyxb.PyXBException(self.message)
        if sys.version_info[:2] > (2, 4):
            self.assertEqual(self.message, e.args[0])

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="str" type="xs:string">
    <xs:annotation><xs:documentation>elt str</xs:documentation></xs:annotation>
  </xs:element>
  <xs:element name="anon">
    <xs:complexType>
      <xs:sequence minOccurs="0" maxOccurs="unbounded">
        <xs:element ref="str"/>
      </xs:sequence>
    </xs:complexType>
    <xs:annotation><xs:documentation>elt anon</xs:documentation></xs:annotation>
  </xs:element>
  <xs:complexType name="cpl">
    <xs:annotation><xs:documentation>ctd cpl</xs:documentation></xs:annotation>
    <xs:sequence minOccurs="0" maxOccurs="unbounded">
      <xs:element name="text" type="xs:string">
        <xs:annotation><xs:documentation>elt cpl.text</xs:documentation></xs:annotation>
      </xs:element>
    </xs:sequence>
  </xs:complexType>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0079 (unittest.TestCase):
    def testDocumentation (self):
        self.assertEqual(str.documentation(), six.u('elt str'))
        self.assertEqual(anon.documentation(), six.u('elt anon'))
        self.assertEqual(cpl.__doc__, six.u('ctd cpl'))
        self.assertEqual(cpl.text.__doc__, six.u('elt cpl.text'))
        self.assertEqual(anon.typeDefinition().__doc__, six.u('elt anon'))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" targetNamespace="urn:trac0191" xmlns="urn:trac0191" attributeFormDefault="qualified">
  <xs:complexType name="tElt">
    <xs:attribute  name="Namespace" type="xs:string"/>
    <xs:attribute  name="More" type="xs:string"/>
  </xs:complexType>
  <xs:element name="elt" type="tElt"/>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

# Note: The generation phase should have printed:
#   Attribute {urn:trac0191}tElt.{urn:trac0191}Namespace renamed to Namespace_
class TestTrac_0191 (unittest.TestCase):
    def testPyxbSymbols (self):
        xmls = '<ns1:elt ns1:Namespace="one" xmlns:ns1="urn:trac0191"/>'
        i = CreateFromDocument(xmls)
        self.assertEqual(i.Namespace_, 'one')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-mg-choice.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils
def ToDOM (instance):
    return instance.toDOM().documentElement

import unittest

class TestMGChoice (unittest.TestCase):
    def onlyFirst (self, instance):
        self.assertTrue(isinstance(instance.first, choice.typeDefinition()._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(instance.second is None)
        self.assertTrue(instance.third is None)

    def onlySecond (self, instance):
        self.assertTrue(instance.first is None)
        self.assertTrue(isinstance(instance.second, choice.typeDefinition()._ElementMap['second'].elementBinding().typeDefinition()))
        self.assertTrue(instance.third is None)

    def onlyThird (self, instance):
        self.assertTrue(instance.first is None)
        self.assertTrue(instance.second is None)
        self.assertTrue(isinstance(instance.third, choice.typeDefinition()._ElementMap['third'].elementBinding().typeDefinition()))

    def testSingleChoice (self):
        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"><first/></ns1:choice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = choice.createFromDOM(dom.documentElement)
        self.onlyFirst(instance)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"><second/></ns1:choice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = choice.createFromDOM(dom.documentElement)
        self.onlySecond(instance)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"><third/></ns1:choice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = choice.createFromDOM(dom.documentElement)
        self.onlyThird(instance)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMissingSingle (self):
        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"/>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, choice.createFromDOM, dom.documentElement)

    def testTooManySingle (self):
        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"><first/><second/></ns1:choice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, choice.createFromDOM, dom.documentElement)

        xmlt = six.u('<ns1:choice xmlns:ns1="URN:test-mg-choice"><second/><third/></ns1:choice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, choice.createFromDOM, dom.documentElement)

    def testMultichoice (self):
        xmlt = six.u('<ns1:multiplechoice xmlns:ns1="URN:test-mg-choice"/>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = multiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(0, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(0, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:multiplechoice xmlns:ns1="URN:test-mg-choice"><first/></ns1:multiplechoice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = multiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(1, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(0, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:multiplechoice xmlns:ns1="URN:test-mg-choice"><first/><first/><first/><third/></ns1:multiplechoice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = multiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(3, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(1, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMultichoiceOrderImportant (self):
        xmlt = six.u('<ns1:multiplechoice xmlns:ns1="URN:test-mg-choice"><first/><third/><first/></ns1:multiplechoice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = multiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(2, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(1, len(instance.third))
        # @todo This test will fail because both firsts will precede the second.
        #self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)


    def testAltMultichoice (self):
        xmlt = six.u('<ns1:altmultiplechoice xmlns:ns1="URN:test-mg-choice"/>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = altmultiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(0, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(0, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:altmultiplechoice xmlns:ns1="URN:test-mg-choice"><first/></ns1:altmultiplechoice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = altmultiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(1, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(0, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

        xmlt = six.u('<ns1:altmultiplechoice xmlns:ns1="URN:test-mg-choice"><first/><first/><third/></ns1:altmultiplechoice>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = altmultiplechoice.createFromDOM(dom.documentElement)
        self.assertEqual(2, len(instance.first))
        self.assertEqual(0, len(instance.second))
        self.assertEqual(1, len(instance.third))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testTooManyChoices (self):
        xmlt = six.u('<ns1:altmultiplechoice xmlns:ns1="URN:test-mg-choice"><first/><first/><first/><third/></ns1:altmultiplechoice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, altmultiplechoice.createFromDOM, dom.documentElement)

    def testFixedMultichoice (self):
        xmlt = six.u('<fixedMultichoice xmlns="URN:test-mg-choice"></fixedMultichoice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = fixedMultichoice.createFromDOM(dom.documentElement)
        xmlt = six.u('<ns1:fixedMultichoice xmlns:ns1="URN:test-mg-choice"><A/><A/></ns1:fixedMultichoice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = fixedMultichoice.createFromDOM(dom.documentElement)
        xmlt = six.u('<ns1:fixedMultichoice xmlns:ns1="URN:test-mg-choice"><A/><B/></ns1:fixedMultichoice>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, fixedMultichoice.createFromDOM, dom.documentElement)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="B" type="xs:byte"/>
  <xs:element name="I" type="xs:int"/>
  <xs:element name="D" type="xs:double"/>
  <xs:group name="types">
     <xs:choice maxOccurs="unbounded">
        <xs:element ref="B"/>
        <xs:element ref="I"/>
        <xs:element ref="D"/>
     </xs:choice>
  </xs:group>
  <xs:complexType name="tData">
    <xs:sequence>
      <xs:group ref="types"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="data" type="tData"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_200908041715 (unittest.TestCase):
    # Verify that we can reconstruct the element associated with
    # content values.

    def testKeyword (self):
        instance = data(B=4)
        self.assertEqual(1, len(instance.orderedContent()))
        cv = instance.orderedContent()[0]
        self.assertEqual(4, cv.value)
        self.assertEqual(cv.value._element(), tData._ElementMap['B'].elementBinding())

        instance = data(I=4)
        self.assertEqual(1, len(instance.orderedContent()))
        cv = instance.orderedContent()[0]
        self.assertEqual(4, cv.value)
        self.assertEqual(cv.value._element(), tData._ElementMap['I'].elementBinding())

    def testValue (self):
        instance = data(4)
        self.assertEqual(1, len(instance.orderedContent()))
        cv = instance.orderedContent()[0]
        self.assertEqual(cv.value._element(), tData._ElementMap['B'].elementBinding())

        instance = data(300)
        self.assertEqual(1, len(instance.orderedContent()))
        cv = instance.orderedContent()[0]
        self.assertEqual(cv.value._element(), tData._ElementMap['I'].elementBinding())

        instance = data(42.3)
        self.assertEqual(1, len(instance.orderedContent()))
        cv = instance.orderedContent()[0]
        self.assertEqual(cv.value._element(), tData._ElementMap['D'].elementBinding())

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import time
from pyxb.utils.six.moves.urllib import request as urllib_request
import weather   # Bindings generated by PyXB
import pyxb.utils.domutils as domutils

uri = 'http://wsf.cdyne.com/WeatherWS/Weather.asmx/GetCityForecastByZIP?ZIP=55113'
xml = urllib_request.urlopen(uri).read()
doc = domutils.StringToDOM(xml)
fc_return = weather.CreateFromDOM(doc.documentElement)
if fc_return.Success:
    print('Weather forecast for %s, %s:' % (fc_return.City, fc_return.State))
    for fc in fc_return.ForecastResult.Forecast:
        when = time.strftime('%A, %B %d %Y', fc.Date.timetuple())
        outlook = fc.Desciption # typos in WSDL left unchanged
        low = fc.Temperatures.MorningLow
        high = fc.Temperatures.DaytimeHigh
        print('  %s: %s, from %s to %s' % (when, outlook, low, high))



# Kenozooid - software stack to support different capabilities of dive
# computers.
#
# Copyright (C) 2009 by Artur Wroblewski &lt;wrobell@pld-linux.org&gt;
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see &lt;http://www.gnu.org/licenses/&gt;.
#
# Modifications by PAB to override reference text

from __future__ import print_function
import os.path
from docutils import nodes
import sys
import re

__Reference_re = re.compile('\s*(.*)\s+<(.*)>\s*$', re.MULTILINE + re.DOTALL)

def api_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    Role `:api:` bridges generated API documentation by tool like EpyDoc
    with Sphinx Python Documentation Generator.

    Other tools, other than EpyDoc, can be easily supported as well.

    First generate the documentation to be referenced, i.e. with EpyDoc::

        $ mkdir -p doc/html/api
        $ epydoc -o doc/html/api ...

    Next step is to generate documentation with Sphinx::

        $ sphinx-build doc doc/html

    """
    basedir = 'api'
    prefix = 'html/' # fixme: fetch it from configuration
    exists = lambda f: os.path.exists(prefix + f)

    # assume module is references

    #print 'Text "%s"' % (text,)
    mo = __Reference_re.match(text)
    label = None
    if mo is not None:
        ( label, text ) = mo.group(1, 2)
    name = text.strip()

    uri = file = '%s/%s-module.html' % (basedir, text)
    chunks = text.split('.')

    #print 'Trying module file %s' % (file,)

    # if not module, then a class
    if not exists(file):
        name = text.split('.')[-1]
        uri = file = '%s/%s-class.html' % (basedir, text)
    #print 'Trying class file %s' % (file,)

    # if not a class, then function or class method
    if not exists(file):
        method = chunks[-1]
        fprefix = '.'.join(chunks[:-1])
        # assume function is referenced
        file = '%s/%s-module.html' % (basedir, fprefix)
        #print 'Trying method file %s' % (file,)
        if exists(file):
            uri = '%s#%s' % (file, method)
        else:
            # class method is references
            file = '%s/%s-class.html' % (basedir, fprefix)
            if exists(file):
                name = '.'.join(chunks[-2:]) # name should be Class.method
                uri = '%s/%s-class.html#%s' % (basedir, fprefix, method)

    if label is None:
        label = name
    if exists(file):
        node = nodes.reference(rawtext, label, refuri=uri, **options)
    else:
        # cannot find reference, then just inline the text
        print('WARNING: Unable to find %s in API' % (text,))
        node = nodes.literal(rawtext, text)

    return [node], []

def ticket_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    Role `:ticket:` generates references to SourceForge tickets.
    """
    trac_root = 'https://sourceforge.net/p/pyxb/tickets'

    # assume module is references

    #print 'Text "%s"' % (text,)
    mo = __Reference_re.match(text)
    label = None
    if mo is not None:
        ( label, text ) = mo.group(1, 2)
    ticket = text.strip()

    uri = '%s/%s/' % (trac_root, ticket)
    if label is None:
        label = 'SF ticket %s' % (ticket,)
    node = nodes.reference(rawtext, label, refuri=uri, **options)

    return [node], []

def issue_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    Role `:issue:` generates references to github issues.
    """
    issue_root = 'https://github.com/pabigot/pyxb/issues'

    # assume module is references

    mo = __Reference_re.match(text)
    label = None
    if mo is not None:
        ( label, text ) = mo.group(1, 2)
    ticket = text.strip()

    uri = '%s/%s/' % (issue_root, ticket)
    if label is None:
        label = 'issue %s' % (ticket,)
    node = nodes.reference(rawtext, label, refuri=uri, **options)

    return [node], []


def pyex_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    Role `:pyex:` generates reference to Python exception classes.
    """

    pyex_fmt = 'http://docs.python.org/library/exceptions.html#exceptions.%s'
    mo = __Reference_re.match(text)
    label = None
    if mo is not None:
        ( label, text ) = mo.group(1, 2)
    exc = text.strip()
    print('Python exception %s as %s' % (text, label))

    uri = pyex_fmt % (exc,)
    if label is None:
        label = '%s' % (exc,)
    node = nodes.reference(rawtext, label, refuri=uri, **options)

    return [node], []

def setup(app):
    app.add_role('api', api_role)
    app.add_config_value('epydoc_basedir', 'api', False)
    app.add_config_value('epydoc_prefix', 'doc/html/', False)
    app.add_role('ticket', ticket_role)
    app.add_role('issue', issue_role)
    app.add_role('pyex', pyex_role)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.xmlschema.structures
import pyxb.utils.domutils

from pyxb.exceptions_ import *

import unittest

def CreateDocumentationNode (content):
    xmls = '<xs:annotation xmlns:xs="%s"><xs:documentation>%s</xs:documentation></xs:annotation>' % (pyxb.namespace.XMLSchema.uri(), content)
    dom = pyxb.utils.domutils.StringToDOM(xmls)
    node = dom.documentElement
    nsc = pyxb.namespace.NamespaceContext.GetNodeContext(node)
    if nsc.targetNamespace() is None:
        nsc.finalizeTargetNamespace()
    return pyxb.xmlschema.structures.Annotation.CreateFromDOM(node)


class TestTrac_0045 (unittest.TestCase):
    def testSimple (self):
        self.assertEqual('hi there!', CreateDocumentationNode("hi there!").asDocString())
        self.assertEqual(' "hi there!" ', CreateDocumentationNode('"hi there!"').asDocString())
        self.assertEqual("''' docstring! '''", CreateDocumentationNode('""" docstring! """').asDocString())
        self.assertEqual("inner ''' docstring!", CreateDocumentationNode('inner """ docstring!').asDocString())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Functions that support activities related to the Document Object Model."""

import logging
import xml.dom

import pyxb
import pyxb.namespace
import pyxb.namespace.resolution
import pyxb.utils.saxutils
import pyxb.utils.saxdom
from pyxb.utils import six
from pyxb.utils.six.moves import xrange

_log = logging.getLogger(__name__)

# The DOM implementation to be used for all processing.  Default is whatever
# your Python install uses, as long as it supports Core 2.0 (for
# createDocument) and XML 2.0 (for NS-aware attribute manipulation).  The
# built-in minidom works fine.
__DOMImplementation = xml.dom.getDOMImplementation(None, (('core', '2.0'), ('xml', '2.0')))

def GetDOMImplementation ():
    """Return the DOMImplementation object used for pyxb operations.

    This is primarily used as the default implementation when generating DOM
    trees from a binding instance.  It defaults to whatever
    xml.dom.getDOMImplementation() returns in your installation (often
    xml.dom.minidom).  It can be overridden with SetDOMImplementation()."""

    global __DOMImplementation
    return __DOMImplementation

def SetDOMImplementation (dom_implementation):
    """Override the default DOMImplementation object."""
    global __DOMImplementation
    __DOMImplementation = dom_implementation
    return __DOMImplementation

# Unfortunately, the DOMImplementation interface doesn't provide a parser.  So
# abstract this in case somebody wants to substitute a different one.  Haven't
# decided how to express that yet.
def StringToDOM (xml_text, **kw):
    """Convert string to a DOM instance.

    @see: L{pyxb._SetXMLStyle}."""

    xmlt = xml_text
    if pyxb.XMLStyle_minidom == pyxb._XMLStyle:
        parser = pyxb.utils.saxutils.make_parser()
        # minidom.parseString is broken.  In Python 2, this means don't
        # feed it unicode.  In Python 3 this means don't feed it bytes.
        if (six.PY2 and not isinstance(xmlt, six.binary_type)):
            xmlt = xmlt.encode(pyxb._InputEncoding)
        elif (six.PY3 and isinstance(xmlt, six.binary_type)):
            xmlt = xmlt.decode(pyxb._InputEncoding)
        return xml.dom.minidom.parseString(xmlt, parser)
    return pyxb.utils.saxdom.parseString(xml_text, **kw)

def NodeAttribute (node, attribute_ncname, attribute_ns=None):
    """Namespace-aware search for an optional attribute in a node.

    @param attribute_ncname: The local name of the attribute.
    @type attribute_ncname: C{str} or C{unicode}

    @keyword attribute_ns: The namespace of the attribute.  Defaults to None
    since most attributes are not in a namespace.  Can be provided as either a
    L{pyxb.namespace.Namespace} instance, or a string URI.
    @type attribute_ns: C{None} or C{str} or C{unicode} or L{pyxb.namespace.Namespace}

    @return: The value of the attribute, or C{None} if the attribute is not
    present.  (Unless C{None}, the value will always be a (unicode) string.)
    """

    ns_uri = attribute_ns
    if isinstance(attribute_ns, pyxb.namespace.Namespace):
        ns_uri = attribute_ns.uri()
    attr = node.getAttributeNodeNS(ns_uri, attribute_ncname)
    if attr is None:
        return None
    return attr.value

def NodeAttributeQName (node, attribute_ncname, attribute_ns=None):
    """Like L{NodeAttribute} but where the content is a QName that must be
    resolved in the context of the node.

    @param attribute_ncname: as in L{NodeAttribute}
    @keyword attribute_ns: as in L{NodeAttribute}

    @return: The expanded name to which the value of the attribute resolves
    given current namespaces, or C{None} if the attribute is not present
    @rtype: L{pyxb.namespace.ExpandedName}
    """
    attr = NodeAttribute(node, attribute_ncname, attribute_ns)
    if attr is None:
        return None
    nsc = pyxb.namespace.NamespaceContext.GetNodeContext(node)
    return nsc.interpretQName(attr)

def LocateUniqueChild (node, tag, absent_ok=True, namespace=pyxb.namespace.XMLSchema):
    """Locate a unique child of the DOM node.

    This function returns the sole child of node which is an ELEMENT_NODE
    instance and has a tag consistent with the given tag.  If multiple nodes
    with a matching C{tag} are found, or C{absent_ok} is C{False} and no
    matching tag is found, an exception is raised.

    @param node: An a xml.dom.Node ELEMENT_NODE instance
    @param tag: the NCName of an element in the namespace
    @keyword absent_ok: If C{True} (default), C{None} is returned if no match
    can be found.  If C{False}, an exception is raised if no match can be
    found.
    @keyword namespace: The namespace to which the child element belongs.
    Default is the XMLSchema namespace.
    @rtype: C{xml.dom.Node}

    @raise pyxb.SchemaValidationError: multiple elements are identified
    @raise pyxb.SchemaValidationError: C{absent_ok} is C{False} and no element is identified.
    """
    candidate = None
    for cn in node.childNodes:
        if (xml.dom.Node.ELEMENT_NODE == cn.nodeType) and namespace.nodeIsNamed(cn, tag):
            if candidate:
                raise pyxb.SchemaValidationError('Multiple %s elements nested in %s' % (tag, node.nodeName))
            candidate = cn
    if (candidate is None) and not absent_ok:
        raise pyxb.SchemaValidationError('Expected %s elements nested in %s' % (tag, node.nodeName))
    return candidate

def LocateMatchingChildren (node, tag, namespace=pyxb.namespace.XMLSchema):
    """Locate all children of the DOM node that have a particular tag.

    This function returns a list of children of node which are ELEMENT_NODE
    instances and have a tag consistent with the given tag.

    @param node: An a xml.dom.Node ELEMENT_NODE instance.
    @param tag: the NCName of an element in the namespace, which defaults to the
    XMLSchema namespace.
    @keyword namespace: The namespace to which the child element belongs.
    Default is the XMLSchema namespace.

    @rtype: C{list(xml.dom.Node)}
    """
    matches = []
    for cn in node.childNodes:
        if (xml.dom.Node.ELEMENT_NODE == cn.nodeType) and namespace.nodeIsNamed(cn, tag):
            matches.append(cn)
    return matches

def LocateFirstChildElement (node, absent_ok=True, require_unique=False, ignore_annotations=True):
    """Locate the first element child of the node.


    @param node: An a xml.dom.Node ELEMENT_NODE instance.
    @keyword absent_ok: If C{True} (default), C{None} is returned if no match
    can be found.  If C{False}, an exception is raised if no match can be
    found.
    @keyword require_unique: If C{False} (default), it is acceptable for there
    to be multiple child elements.  If C{True}, presence of multiple child
    elements raises an exception.
    @keyword ignore_annotations: If C{True} (default), annotations are skipped
    wheen looking for the first child element.  If C{False}, an annotation
    counts as an element.
    @rtype: C{xml.dom.Node}

    @raise SchemaValidationError: C{absent_ok} is C{False} and no child
    element was identified.
    @raise SchemaValidationError: C{require_unique} is C{True} and multiple
    child elements were identified
    """

    candidate = None
    for cn in node.childNodes:
        if xml.dom.Node.ELEMENT_NODE == cn.nodeType:
            if ignore_annotations and pyxb.namespace.XMLSchema.nodeIsNamed(cn, 'annotation'):
                continue
            if require_unique:
                if candidate:
                    raise pyxb.SchemaValidationError('Multiple elements nested in %s' % (node.nodeName,))
                candidate = cn
            else:
                return cn
    if (candidate is None) and not absent_ok:
        raise pyxb.SchemaValidationError('No elements nested in %s' % (node.nodeName,))
    return candidate

def HasNonAnnotationChild (node):
    """Return True iff C{node} has an ELEMENT_NODE child that is not an
    XMLSchema annotation node.

    @rtype: C{bool}
    """
    for cn in node.childNodes:
        if (xml.dom.Node.ELEMENT_NODE == cn.nodeType) and (not pyxb.namespace.XMLSchema.nodeIsNamed(cn, 'annotation')):
            return True
    return False

def ExtractTextContent (node):
    """Walk all the children, extracting all text content and
    catenating it into the return value.

    Returns C{None} if no text content (including whitespace) is found.

    This is mainly used to strip comments out of the content of complex
    elements with simple types.

    @rtype: C{unicode} or C{str}
    """
    text = []
    for cn in node.childNodes:
        if xml.dom.Node.TEXT_NODE == cn.nodeType:
            text.append(cn.data)
        elif xml.dom.Node.CDATA_SECTION_NODE == cn.nodeType:
            text.append(cn.data)
        elif xml.dom.Node.COMMENT_NODE == cn.nodeType:
            pass
        else:
            raise pyxb.NonElementValidationError(cn)
    if 0 == len(text):
        return None
    return ''.join(text)

class BindingDOMSupport (object):
    """This holds DOM-related information used when generating a DOM tree from
    a binding instance."""

    def implementation (self):
        """The DOMImplementation object to be used.

        Defaults to L{pyxb.utils.domutils.GetDOMImplementation()}, but can be
        overridden in the constructor call using the C{implementation}
        keyword."""
        return self.__implementation
    __implementation = None

    def document (self):
        """Return the document generated using this instance."""
        return self.__document
    __document = None

    def requireXSIType (self):
        """Indicates whether {xsi:type<http://www.w3.org/TR/xmlschema-1/#xsi_type>} should be added to all elements.

        Certain WSDL styles and encodings seem to require explicit notation of
        the type of each element, even if it was specified in the schema.

        This value can only be set in the constructor."""
        return self.__requireXSIType
    __requireXSIType = None

    def reset (self):
        """Reset this instance to the state it was when created.

        This creates a new root document with no content, resets the
        namespace-prefix map to its as-constructed content, and clears the set
        of referenced namespace prefixes.  The defaultNamespace and
        requireXSIType are not modified."""
        self.__document = self.implementation().createDocument(None, None, None)
        self.__namespaceContext.reset()
        # For historical reasons this is also added automatically, though
        # 'xsi' is not a bound prefix.
        self.__namespaceContext.declareNamespace(pyxb.namespace.XMLSchema_instance, 'xsi')
        self.__referencedNamespacePrefixes = set()

    @classmethod
    def Reset (cls):
        """Reset the global defaults for default/prefix/namespace information."""
        cls.__NamespaceContext.reset()

    def __init__ (self, implementation=None, default_namespace=None, require_xsi_type=False, namespace_prefix_map=None):
        """Create a new instance used for building a single document.

        @keyword implementation: The C{xml.dom} implementation to use.
        Defaults to the one selected by L{GetDOMImplementation}.

        @keyword default_namespace: The namespace to configure as the default
        for the document.  If not provided, there is no default namespace.
        @type default_namespace: L{pyxb.namespace.Namespace}

        @keyword require_xsi_type: If C{True}, an U{xsi:type
        <http://www.w3.org/TR/xmlschema-1/#xsi_type>} attribute should be
        placed in every element.
        @type require_xsi_type: C{bool}

        @keyword namespace_prefix_map: A map from pyxb.namespace.Namespace
        instances to the preferred prefix to use for the namespace in xmlns
        declarations.  The default one assigns 'xsi' for the XMLSchema
        instance namespace.
        @type namespace_prefix_map: C{map} from L{pyxb.namespace.Namespace} to C{str}

        @raise pyxb.LogicError: the same prefix is associated with multiple
        namespaces in the C{namespace_prefix_map}.

        """
        if implementation is None:
            implementation = GetDOMImplementation()
        self.__implementation = implementation
        self.__requireXSIType = require_xsi_type
        self.__namespaceContext = pyxb.namespace.NamespaceContext(parent_context=self.__NamespaceContext,
                                                                  in_scope_namespaces=namespace_prefix_map)
        if default_namespace is not None:
            self.__namespaceContext.setDefaultNamespace(default_namespace)
        self.reset()

    # Default namespace-prefix map support
    __NamespaceContext = pyxb.namespace.NamespaceContext()

    # Instance-specific namespace-prefix map support
    __namespaceContext = None

    # Set of pairs of (namespace, prefix) identifying the declarations that
    # must be placed in the document root so that QNames can be resolved.
    # These are the prefixes associated with namespaces that were queried
    # through L{namespacePrefix()} since the last reset().
    __referencedNamespacePrefixes = None

    def defaultNamespace (self):
        """The default namespace for this instance"""
        return self.__namespaceContext.defaultNamespace()
    @classmethod
    def DefaultNamespace (cls):
        """The global default namespace (used on instance creation if not overridden)"""
        return cls.__NamespaceContext.defaultNamespace()

    def setDefaultNamespace (self, default_namespace):
        return self.__namespaceContext.setDefaultNamespace(default_namespace)
    @classmethod
    def SetDefaultNamespace (cls, default_namespace):
        return cls.__NamespaceContext.setDefaultNamespace(default_namespace)

    def declareNamespace (self, namespace, prefix=None):
        """Declare a namespace within this instance only."""
        return self.__namespaceContext.declareNamespace(namespace, prefix)
    @classmethod
    def DeclareNamespace (cls, namespace, prefix=None):
        """Declare a namespace that will made available to each created instance."""
        return cls.__NamespaceContext.declareNamespace(namespace, prefix)

    def namespacePrefix (self, namespace, enable_default_namespace=True):
        """Return the prefix to be used for the given namespace.

        This will L{declare <declareNamespace>} the namespace if it has not
        yet been observed.  It will also ensure the mapping from the returned
        prefix to C{namespace} is recorded for addition as an xmlns directive
        in the final document.

        @param namespace: The namespace for which a prefix is needed.  If the
        provided namespace is C{None} or an absent namespace, the C{None}
        value will be returned as the corresponding prefix.

        @keyword enable_default_namespace: Normally if the namespace is the default
        namespace C{None} is returned to indicate this.  If this keyword is
        C{False} then we need a namespace prefix even if this is the default.
        """
        if (namespace is None) or namespace.isAbsentNamespace():
            return None
        if isinstance(namespace, six.string_types):
            namespace = pyxb.namespace.NamespaceForURI(namespace, create_if_missing=True)
        if (self.defaultNamespace() == namespace) and enable_default_namespace:
            return None
        pfx = self.__namespaceContext.prefixForNamespace(namespace)
        if pfx is None:
            pfx = self.__namespaceContext.declareNamespace(namespace)
        self.__referencedNamespacePrefixes.add((namespace, pfx))
        return pfx

    def qnameAsText (self, qname, enable_default_namespace=True):
        assert isinstance(qname, pyxb.namespace.ExpandedName)
        name = qname.localName()
        prefix = self.namespacePrefix(qname.namespace(), enable_default_namespace=enable_default_namespace)
        if prefix is not None:
            name = '%s:%s' % (prefix, name)
        return name

    def valueAsText (self, value, enable_default_namespace=True):
        """Represent a simple type value as XML text.

        This is essentially what C{value.xsdLiteral()} does, but this one
        handles any special cases such as QName values where the lexical
        representation cannot be done in isolation of external information
        such as namespace declarations."""
        from pyxb.binding.basis import simpleTypeDefinition, STD_list
        if isinstance(value, pyxb.namespace.ExpandedName):
            return self.qnameAsText(value, enable_default_namespace=enable_default_namespace)
        if isinstance(value, STD_list):
            return ' '.join([ self.valueAsText(_v, enable_default_namespace=enable_default_namespace) for _v in value ])
        if isinstance(value, simpleTypeDefinition):
            return value.xsdLiteral()
        assert value is not None
        return six.text_type(value)

    def addAttribute (self, element, expanded_name, value):
        """Add an attribute to the given element.

        @param element: The element to which the attribute should be added
        @type element: C{xml.dom.Element}
        @param expanded_name: The name of the attribute.  This may be a local
        name if the attribute is not in a namespace.
        @type expanded_name: L{pyxb.namespace.Namespace} or C{str} or C{unicode}
        @param value: The value of the attribute
        @type value: C{str} or C{unicode}
        """
        name = expanded_name
        ns_uri = xml.dom.EMPTY_NAMESPACE
        if isinstance(name, pyxb.namespace.ExpandedName):
            ns_uri = expanded_name.namespaceURI()
            # Attribute names do not use default namespace
            name = self.qnameAsText(expanded_name, enable_default_namespace=False)
        element.setAttributeNS(ns_uri, name, self.valueAsText(value))

    def addXMLNSDeclaration (self, element, namespace, prefix=None):
        """Manually add an XMLNS declaration to the document element.

        @param namespace: a L{pyxb.namespace.Namespace} instance

        @param prefix: the prefix by which the namespace is known.  If
        C{None}, the default prefix as previously declared will be used; if
        C{''} (empty string) a declaration for C{namespace} as the default
        namespace will be generated.

        @return: C{prefix} as used in the added declaration.
        """
        if not isinstance(namespace, pyxb.namespace.Namespace):
            raise pyxb.UsageError('addXMLNSdeclaration: must be given a namespace instance')
        if namespace.isAbsentNamespace():
            raise pyxb.UsageError('addXMLNSdeclaration: namespace must not be an absent namespace')
        if prefix is None:
            prefix = self.namespacePrefix(namespace)
        if not prefix: # None or empty string
            an = 'xmlns'
        else:
            an = 'xmlns:' + prefix
        element.setAttributeNS(pyxb.namespace.XMLNamespaces.uri(), an, namespace.uri())
        return prefix

    def finalize (self):
        """Do the final cleanup after generating the tree.  This makes sure
        that the document element includes XML Namespace declarations for all
        namespaces referenced in the tree.

        @return: The document that has been created.
        @rtype: C{xml.dom.Document}"""
        ns = self.defaultNamespace()
        if ns is not None:
            self.addXMLNSDeclaration(self.document().documentElement, ns, '')
        for (ns, pfx) in self.__referencedNamespacePrefixes:
            self.addXMLNSDeclaration(self.document().documentElement, ns, pfx)
        return self.document()

    def createChildElement (self, expanded_name, parent=None):
        """Create a new element node in the tree.

        @param expanded_name: The name of the element.  A plain string
        indicates a name in no namespace.
        @type expanded_name: L{pyxb.namespace.ExpandedName} or C{str} or C{unicode}

        @keyword parent: The node in the tree that will serve as the child's
        parent.  If C{None}, the document element is used.  (If there is no
        document element, then this call creates it as a side-effect.)

        @return: A newly created DOM element
        @rtype: C{xml.dom.Element}
        """

        if parent is None:
            parent = self.document().documentElement
        if parent is None:
            parent = self.__document
        if isinstance(expanded_name, six.string_types):
            expanded_name = pyxb.namespace.ExpandedName(None, expanded_name)
        if not isinstance(expanded_name, pyxb.namespace.ExpandedName):
            raise pyxb.LogicError('Invalid type %s for expanded name' % (type(expanded_name),))
        ns = expanded_name.namespace()
        ns_uri = xml.dom.EMPTY_NAMESPACE
        name = expanded_name.localName()
        if ns is not None:
            ns_uri = ns.uri()
            name = self.qnameAsText(expanded_name)
        element = self.__document.createElementNS(ns_uri, name)
        return parent.appendChild(element)

    def _makeURINodeNamePair (self, node):
        """Convert namespace information from a DOM node to text for new DOM node.

        The namespaceURI and nodeName are extracted and parsed.  The namespace
        (if any) is registered within the document, along with any prefix from
        the node name.  A pair is returned where the first element is the
        namespace URI or C{None}, and the second is a QName to be used for the
        expanded name within this document.

        @param node: An xml.dom.Node instance, presumably from a wildcard match.
        @rtype: C{( str, str )}"""
        ns = None
        if node.namespaceURI is not None:
            ns = pyxb.namespace.NamespaceForURI(node.namespaceURI, create_if_missing=True)
        if node.ELEMENT_NODE == node.nodeType:
            name = node.tagName
        elif node.ATTRIBUTE_NODE == node.nodeType:
            name = node.name
            # saxdom uses the uriTuple as the name field while minidom uses
            # the QName.  @todo saxdom should be fixed.
            if isinstance(name, tuple):
                name = name[1]
        else:
            raise pyxb.UsageError('Unable to determine name from DOM node %s' % (node,))
        pfx = None
        local_name = name
        if 0 < name.find(':'):
            (pfx, local_name) = name.split(':', 1)
            if ns is None:
                raise pyxb.LogicError('QName with prefix but no available namespace')
        ns_uri = None
        node_name = local_name
        if ns is not None:
            ns_uri = ns.uri()
            self.declareNamespace(ns, pfx)
            node_name = self.qnameAsText(ns.createExpandedName(local_name))
        return (ns_uri, node_name)

    def _deepClone (self, node, docnode):
        if node.ELEMENT_NODE == node.nodeType:
            (ns_uri, node_name) = self._makeURINodeNamePair(node)
            clone_node = docnode.createElementNS(ns_uri, node_name)
            attrs = node.attributes
            for ai in xrange(attrs.length):
                clone_node.setAttributeNodeNS(self._deepClone(attrs.item(ai), docnode))
            for child in node.childNodes:
                clone_node.appendChild(self._deepClone(child, docnode))
            return clone_node
        if node.TEXT_NODE == node.nodeType:
            return docnode.createTextNode(node.data)
        if node.ATTRIBUTE_NODE == node.nodeType:
            (ns_uri, node_name) = self._makeURINodeNamePair(node)
            clone_node = docnode.createAttributeNS(ns_uri, node_name)
            clone_node.value = node.value
            return clone_node
        if node.COMMENT_NODE == node.nodeType:
            return docnode.createComment(node.data)
        raise ValueError('DOM node not supported in clone', node)

    def cloneIntoImplementation (self, node):
        """Create a deep copy of the node in the target implementation.

        Used when converting a DOM instance from one implementation (e.g.,
        L{pyxb.utils.saxdom}) into another (e.g., L{xml.dom.minidom})."""
        new_doc = self.implementation().createDocument(None, None, None)
        return self._deepClone(node, new_doc)

    def appendChild (self, child, parent):
        """Add the child to the parent.

        @note: If the child and the parent use different DOM implementations,
        this operation will clone the child into a new instance, and give that
        to the parent.

        @param child: The value to be appended
        @type child: C{xml.dom.Node}
        @param parent: The new parent of the child
        @type parent: C{xml.dom.Node}
        @rtype: C{xml.dom.Node}"""

        # @todo This check is incomplete; is there a standard way to find the
        # implementation of an xml.dom.Node instance?
        if isinstance(child, (pyxb.utils.saxdom.Node, xml.dom.minidom.Node)):
            child = self.cloneIntoImplementation(child)
        return parent.appendChild(child)

    def appendTextChild (self, text, parent):
        """Add the text to the parent as a text node."""
        return parent.appendChild(self.document().createTextNode(self.valueAsText(text)))

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="f13p8">
    <xs:restriction base="xs:decimal">
      <xs:totalDigits value="13"/>
      <xs:fractionDigits value="8"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:element name="e13p8" type="f13p8"/>
  <xs:simpleType name="f15p5">
    <xs:restriction base="xs:decimal">
      <xs:totalDigits value="15"/>
      <xs:fractionDigits value="5"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:element name="e15p5" type="f15p5"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0091 (unittest.TestCase):
    def assertAlmostEqual (self, v1, v2, *args, **kw):
        from decimal import Decimal
        if (isinstance(v1, Decimal)
            or isinstance(v2, Decimal)):
            if not isinstance(v1, Decimal):
                v1 = Decimal(str(v1))
            if not isinstance(v2, Decimal):
                v2 = Decimal(str(v2))
        return super(TestTrac_0091, self).assertAlmostEqual(v1, v2, *args, **kw)

    def testBasic (self):
        if sys.version_info[:2] >= (2, 7):
            # Prior to 2.7 float/Decimal comparisons returned invalid results.
            self.assertEqual(1.0, e13p8(1.0))
            self.assertEqual(1.0, e13p8('1.0'))
            self.assertEqual(1234567890123.0, e13p8('1234567890123'))
            self.assertEqual(1234567890123.0, CreateFromDocument('<e13p8>1234567890123</e13p8>'))
        self.assertRaises(SimpleFacetValueError, e13p8, '12345678901234')
        self.assertAlmostEqual(1.00000001, e13p8('1.00000001'))
        self.assertRaises(SimpleFacetValueError, e13p8, '1.000000001')

    def testBadCase (self):
        # Prior to fix, this raised a facet violation due to rounding
        self.assertAlmostEqual(0.00790287, e13p8('0.00790287'))

    def test15_5 (self):
        from decimal import Decimal
        # For compatibility in Python 2.6 PyXB will convert the float
        # to a string before invoking the underlying decimal.Decimal
        fv = e15p5(1234.56789)
        self.assertTrue(fv.validateBinding())
        dv = Decimal('1234.56789')
        self.assertEqual(fv, dv)
        # In Python 2.7 decimal.Decimal will create from the float,
        # which will already be in a form that won't validate
        if sys.version_info[:2] >= (2, 7):
            self.assertRaises(SimpleFacetValueError, e15p5, Decimal(1234.56789))
        sv = e15p5('1234.56789')
        self.assertEqual(sv, dv)
        self.assertTrue(sv.validateBinding())
        self.assertTrue(e15p5('1000000.0').validateBinding())

    def testRanges (self):
        from decimal import Decimal
        o14o = [1] + ([0] * 12) + [1]
        self.assertEqual(14, len(o14o))
        o15o = [1] + ([0] * 13) + [1]
        self.assertEqual(15, len(o15o))
        o16o = [1] + ([0] * 14) + [1]
        self.assertEqual(16, len(o16o))

        self.assertTrue(e15p5(Decimal((0, o14o, 0))).validateBinding())
        self.assertTrue(e15p5(Decimal((0, o15o, 0))).validateBinding())

        # Negative exponents do not reduce total digit count
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            e15p5(Decimal((0, o16o, 0)))
        self.assertEqual(cm.exception.facet, f15p5._CF_totalDigits)
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            e15p5(Decimal((0, o16o, -1)))
        self.assertEqual(cm.exception.facet, f15p5._CF_totalDigits)

        # Positive exponents add to total digit count
        self.assertTrue(e15p5(Decimal((0, o14o, 1))).validateBinding())
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            e15p5(Decimal((0, o15o, 1)))
        self.assertEqual(cm.exception.facet, f15p5._CF_totalDigits)
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            e15p5(Decimal((0, o14o, 2)))
        self.assertEqual(cm.exception.facet, f15p5._CF_totalDigits)

        # Negative exponents affect fractionDigits only
        self.assertTrue(e15p5(Decimal((0, o15o, -1))).validateBinding())
        self.assertTrue(e15p5(Decimal((0, o15o, -5))).validateBinding())
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            e15p5(Decimal((0, o15o, -6)))
        self.assertEqual(cm.exception.facet, f15p5._CF_fractionDigits)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" targetNamespace="urn:trac-0208">
  <xs:complexType name="BaseComplex">
    <xs:complexContent mixed="false">
      <xs:extension base="xs:anyType"/>
    </xs:complexContent>
  </xs:complexType>
  <xs:complexType name="ChildComplex">
    <xs:complexContent mixed="false">
      <xs:extension xmlns:tns="urn:trac-0208" base="tns:BaseComplex"/>
    </xs:complexContent>
  </xs:complexType>
  <xs:simpleType name="BaseSimple">
    <xs:restriction base="xs:int"/>
  </xs:simpleType>
  <xs:simpleType name="ChildSimple">
    <xs:restriction xmlns:tns="urn:trac-0208" base="tns:BaseSimple"/>
  </xs:simpleType>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0208 (unittest.TestCase):

    def testBasic (self):
        pass

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import sys
import time
from pyxb.utils.six.moves.urllib import request as urllib_request
import pyxb.bundles.wssplat.soap11 as soapenv
import weather

zip = 55113
if 1 < len(sys.argv):
    zip = int(sys.argv[1])

# Create an envelope, and give it a body that is the request for the
# service we want.
env = soapenv.Envelope(soapenv.Body(weather.GetCityForecastByZIP(ZIP=str(zip))))
open('request.xml', 'w').write(env.toxml("utf-8"))

# Invoke the service
uri = urllib_request.Request('http://wsf.cdyne.com/WeatherWS/Weather.asmx',
                             env.toxml("utf-8"),
                             { 'SOAPAction' : "http://ws.cdyne.com/WeatherWS/GetCityForecastByZIP", 'Content-Type': 'text/xml' } )
rxml = urllib_request.urlopen(uri).read()
open('response.xml', 'w').write(rxml)

# Convert the response to a SOAP envelope, then extract the actual
# response from the wildcard elements of the body.  Note that because
# the weather namespace was registered, PyXB already created the
# binding for the response.
soap_resp = soapenv.CreateFromDocument(rxml)
resp = soap_resp.Body.wildcardElements()[0]

fc_return = resp.GetCityForecastByZIPResult
if fc_return.Success:
    print('Got response for %s, %s:' % (fc_return.City, fc_return.State))
    for fc in fc_return.ForecastResult.Forecast:
        when = time.strftime('%A, %B %d %Y', fc.Date.timetuple())
        outlook = fc.Desciption # typos in WSDL left unchanged
        low = fc.Temperatures.MorningLow
        high = fc.Temperatures.DaytimeHigh
        print('  %s: %s, from %s to %s' % (when, outlook, low, high))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node
import pyxb.namespace

import os.path
xst = '''<?xml version="1.0"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="var_test">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="data" type="xs:anyType"/>
      </xs:sequence>
   </xs:complexType>
  </xs:element>
  <xs:simpleType name="tSimple">
     <xs:restriction base="xs:string"/>
  </xs:simpleType>
  <xs:complexType name="tComplex">
    <xs:sequence>
      <xs:element name="logstring">
        <xs:simpleType>
          <xs:restriction base="xs:string"/>
        </xs:simpleType>
      </xs:element>
    </xs:sequence>
  </xs:complexType>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIssue15 (unittest.TestCase):
    def testSimple (self):
        vt = var_test()
        vt.data = tSimple("s")
        instance = CreateFromDOM(vt.toDOM())
        self.assertIsInstance(instance.data, tSimple)

    def testComplex (self):
        vt = var_test()
        vt.data = tComplex("c")
        instance = CreateFromDOM(vt.toDOM())
        self.assertIsInstance(instance.data, tComplex)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd=six.u('''<?xml version="1.0" encoding="utf-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="anything" type="xs:anyType" nillable="true"/>
        <xs:element name="container">
                <xs:complexType>
                        <xs:sequence>
                                <xs:element ref="anything"/>
                        </xs:sequence>
                </xs:complexType>
        </xs:element>
</xs:schema>
''')

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

import pyxb.utils.domutils
import pyxb.namespace
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(pyxb.namespace.XMLSchema, 'xs')

class TestTrac_0094 (unittest.TestCase):
    body = 'something'
    xmlt = six.u('''<anything xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="xs:string">%s</anything>''') % (body,)
    xmld = xmlt.encode('utf-8')

    def testFromXML (self):
        instance = CreateFromDocument(self.xmlt)
        self.assertTrue(isinstance(instance, xs.string))
        self.assertEqual(instance, self.body)
        self.assertEqual(instance._element(), anything)

    def testToXML (self):
        instance = xs.string(self.body, _element=anything)
        self.assertEqual(instance.toxml("utf-8", root_only=True), self.xmld)

    def testContainerCtor (self):
        i = xs.string(self.body, _element=anything)
        instance = container(anything=i)
        explicit_xml = instance.toxml("utf-8")
        instance = container(anything=xs.string(self.body))
        implicit_xml = instance.toxml("utf-8")
        self.assertEqual(explicit_xml, implicit_xml)

    def testContainerAssignment (self):
        i = xs.string(self.body, _element=anything)
        instance = container()
        instance.anything = i
        explicit_xml = instance.toxml("utf-8")
        instance.anything = xs.string(self.body)
        implicit_xml = instance.toxml("utf-8")
        self.assertEqual(explicit_xml, implicit_xml)
        instance.anything = xs.int(43)
        self.assertTrue(isinstance(instance.anything, xs.int))
        int_xml = instance.toxml("utf-8")
        instance.anything = self.body
        self.assertTrue(isinstance(instance.anything, xs.anyType))
        oc = instance.anything.orderedContent()
        self.assertEqual(1, len(oc))
        self.assertTrue(isinstance(oc[0], pyxb.binding.basis.NonElementContent))
        xmlt = six.u('<container><anything>something</anything></container>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(xmld, instance.toxml('utf-8', root_only=True))
        instance.anything = 43
        self.assertTrue(isinstance(instance.anything, xs.anyType))
        oc = instance.anything.orderedContent()
        self.assertEqual(1, len(oc))
        self.assertTrue(isinstance(oc[0], pyxb.binding.basis.NonElementContent))
        xmlt = six.u('<container><anything>43</anything></container>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(xmld, instance.toxml('utf-8', root_only=True))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xst = '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="tMixed" mixed="true">
    <xs:sequence>
      <xs:element name="mString" type="xs:string"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="Mixed" type="tMixed"/>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0212 (unittest.TestCase):

    def testBasicMixed (self):
        xmlt = six.u('<Mixed><mString>body</mString></Mixed>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        xmlt = six.u('<Mixed>pre<mString>body</mString>post</Mixed>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        oc = instance.orderedContent()
        self.assertEqual(3, len(oc))
        self.assertEqual(six.u('pre'), oc[0].value)
        self.assertEqual(instance.mString, oc[1].value)
        self.assertEqual(six.u('post'), oc[2].value)
        nec = list(pyxb.NonElementContent(instance))
        self.assertEqual(2, len(nec))
        self.assertEqual(nec[0], six.u('pre'))
        self.assertEqual(nec[1], six.u('post'))
        self.assertEqual(six.u('prepost'), ''.join(pyxb.NonElementContent(instance)))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Extensions of standard exceptions for PyXB events.

Yeah, I'd love this module to be named exceptions.py, but it can't
because the standard library has one of those, and we need to
reference it below.
"""

import pyxb
from pyxb.utils import six

class PyXBException (Exception):
    """Base class for exceptions that indicate a problem that the user should fix."""

    """The arguments passed to the exception constructor."""
    _args = None

    """The keywords passed to the exception constructor.

    @note: Do not pop values from the keywords array in subclass
    constructors that recognize and extract values from them.  They
    should be kept around so they're accessible generically."""
    _kw = None

    def __init__ (self, *args, **kw):
        """Create an exception indicating a PyXB-related problem.

        If no args are present, a default argument is taken from the
        C{message} keyword.

        @keyword message : Text to provide the user with information about the problem.
        """
        if 0 == len(args) and 'message' in kw:
            args = (kw.pop('message'),)
        self._args = args
        self._kw = kw
        super(PyXBException, self).__init__(*args)

    if six.PY2:
        def _str_from_unicode (self):
            return unicode(self).encode(pyxb._OutputEncoding)

class PyXBVersionError (PyXBException):
    """Raised on import of a binding generated with a different version of PYXB"""
    pass

class DOMGenerationError (PyXBException):
    """A non-validation error encountered converting bindings to DOM."""
    pass

@six.python_2_unicode_compatible
class UnboundElementError (DOMGenerationError):
    """An instance converting to DOM had no bound element."""

    instance = None
    """The binding instance.  This is missing an element binding (via
    L{pyxb.binding.basis._TypeBinding_mixin._element}) and no
    C{element_name} was passed."""

    def __init__ (self, instance):
        super(UnboundElementError, self).__init__(instance)
        self.instance = instance

    def __str__ (self):
        return six.u('Instance of type %s has no bound element for start tag') % (self.instance._diagnosticName(),)

class SchemaValidationError (PyXBException):
    """Raised when the XML hierarchy does not appear to be valid for an XML schema."""
    pass

class NamespaceError (PyXBException):
    """Violation of some rule relevant to XML Namespaces"""
    def __init__ (self, namespace, *args, **kw):
        PyXBException.__init__(self, *args, **kw)
        self.__namespace = namespace

    def namespace (self): return self.__namespace

class NamespaceArchiveError (PyXBException):
    """Problem related to namespace archives"""
    pass

class SchemaUniquenessError (PyXBException):
    """Raised when somebody tries to create a schema component using a
    schema that has already been used in that namespace.  Import and
    include processing would have avoided this, so somebody asked for
    it specifically."""
    def __init__ (self, namespace, schema_location, existing_schema, *args, **kw):
        super(SchemaUniquenessError, self).__init__(*args, **kw)
        self.__namespace = namespace
        self.__schemaLocation = schema_location
        self.__existingSchema = existing_schema

    def namespace (self): return self.__namespace
    def schemaLocation (self): return self.__schemaLocation
    def existingSchema (self): return self.__existingSchema

class BindingGenerationError (PyXBException):
    """Raised when something goes wrong generating the binding classes"""
    pass

class NamespaceUniquenessError (NamespaceError):
    """Raised when an attempt is made to record multiple objects of the same name in the same namespace category."""
    pass

class NotInNamespaceError (PyXBException):
    '''Raised when a name is referenced that is not defined in the appropriate namespace.'''
    __namespace = None
    __ncName = None

class QNameResolutionError (NamespaceError):
    '''Raised when a QName cannot be associated with a namespace.'''
    namespaceContext = None
    qname = None

    def __init__ (self, message, qname, xmlns_context):
        self.qname = qname
        self.namespaceContext = xmlns_context
        super(QNameResolutionError, self).__init__(message, qname, xmlns_context)

class BadDocumentError (PyXBException):
    """Raised when processing document content and an error is encountered."""
    pass

class StructuralBadDocumentError (BadDocumentError):
    """Raised when processing document and the content model is not satisfied."""
    @property
    def element_use (self):
        """The L{pyxb.binding.content.ElementDeclaration} instance to which the content should conform, if available."""
        return self.__elementUse

    @property
    def container (self):
        """The L{pyxb.binding.basis.complexTypeDefinition} instance to which the content would belong, if available."""
        return self.__container

    @property
    def content (self):
        """The value which could not be reconciled with the content model."""
        return self.__content

    def __init__ (self, *args, **kw):
        """Raised when processing document and the content model is not satisfied.

        @keyword content : The value that could not be reconciled with the content model
        @keyword container : Optional binding instance into which the content was to be assigned
        @keyword element_use : Optional reference to an element use identifying the element to which the value was to be reconciled
        """
        self.__content = kw.pop('content', None)
        if args:
            self.__content = args[0]
        self.__container = kw.pop('container', None)
        self.__elementUse = kw.pop('element_use', None)
        if self.__content is not None:
            if self.__container is not None:
                kw.setdefault('message', '%s cannot accept wildcard content %s' % (self.__container._Name(), self.__content))
            elif self.__elementUse is not None:
                kw.setdefault('message', '%s not consistent with content model for %s' % (self.__content, self.__elementUse))
            else:
                kw.setdefault('message', six.text_type(self.__content))
        BadDocumentError.__init__(self, **kw)

class UnrecognizedDOMRootNodeError (StructuralBadDocumentError):
    """A root DOM node could not be resolved to a schema element"""

    node = None
    """The L{xml.dom.Element} instance that could not be recognized"""

    def __get_node_name (self):
        """The QName of the L{node} as a L{pyxb.namespace.ExpandedName}"""
        import pyxb.namespace
        return  pyxb.namespace.ExpandedName(self.node.namespaceURI, self.node.localName)
    node_name = property(__get_node_name)

    def __init__ (self, node):
        """@param node: the value for the L{node} attribute."""
        self.node = node
        super(UnrecognizedDOMRootNodeError, self).__init__(node)

class ValidationError (PyXBException):
    """Raised when something in the infoset fails to satisfy a content model or attribute requirement.

    All validation errors include a L{location} attribute which shows
    where in the original XML the problem occurred.  The attribute may
    be C{None} if the content did not come from an XML document, or
    the underlying XML infrastructure did not provide a location.

    More refined validation error exception classes add more attributes."""

    location = None
    """Where the error occurred in the document being parsed, if
    available.  This will be C{None}, or an instance of
    L{pyxb.utils.utility.Location}."""

    def details (self):
        """Provide information describing why validation failed.

        In many cases, this is simply the informal string content that
        would be obtained through the C{str} built-in function.  For
        certain errors this method gives more details on what would be
        acceptable and where the descriptions can be found in the
        original schema.

        @return: a string description of validation failure"""
        return six.text_type(self)

@six.python_2_unicode_compatible
class NonElementValidationError (ValidationError):
    """Raised when an element (or a value bound to an element) appears
    in context that does not permit an element."""

    element = None
    """The content that is not permitted.  This may be an element, or
    a DOM representation that would have been made into an element had
    matters progressed further."""

    def __init__ (self, element, location=None):
        """@param element: the value for the L{element} attribute.
        @param location: the value for the L{location} attribute.
        """
        self.element = element
        if (location is None) and isinstance(element, pyxb.utils.utility.Locatable_mixin):
            location = element._location()
        self.location = location
        super(NonElementValidationError, self).__init__(element, location)

    def __str__ (self):
        import pyxb.binding.basis
        import xml.dom
        value = ''
        boundto = ''
        location = ''
        if isinstance(self.element, pyxb.binding.basis._TypeBinding_mixin):
            eb = self.element._element()
            boundto = ''
            if eb is not None:
                boundto = ' bound to %s' % (eb.name(),)
            if isinstance(self.element, pyxb.binding.basis.simpleTypeDefinition):
                value = self.element.xsdLiteral()
            elif self.element._IsSimpleTypeContent():
                value = six.text_type(self.element.value())
            else:
                value = 'Complex value'
        elif isinstance(self.element, xml.dom.Node):
            value = 'DOM node %s' % (self.element.nodeName,)
        else:
            value = '%s type %s' % (six.text_type(self.element), type(self.element))
        if self.location is not None:
            location = ' at %s' % (self.location,)
        return six.u('%s%s not permitted%s') % (value, boundto, location)

class ElementValidationError (ValidationError):
    """Raised when a validation requirement for an element is not satisfied."""
    pass

@six.python_2_unicode_compatible
class AbstractElementError (ElementValidationError):
    """Attempt to create an instance of an abstract element.

    Raised when an element is created and the identified binding is
    abstract.  Such elements cannot be created directly; instead the
    creation must derive from an instance of the abstract element's
    substitution group.

    Since members of the substitution group self-identify using the
    C{substitutionGroup} attribute, there is no general way to find
    the set of elements which would be acceptable in place of the
    abstract element."""

    element = None
    """The abstract L{pyxb.binding.basis.element} in question"""

    value = None
    """The value proposed for the L{element}.  This is usually going
    to be a C{xml.dom.Node} used in the attempt to create the element,
    C{None} if the abstract element was invoked without a node, or
    another type if
    L{pyxb.binding.content.ElementDeclaration.toDOM} is
    mis-used."""

    def __init__ (self, element, location, value=None):
        """@param element: the value for the L{element} attribute.
        @param location: the value for the L{location} attribute.
        @param value: the value for the L{value} attribute."""
        self.element = element
        self.location = location
        self.value = value
        super(AbstractElementError, self).__init__(element, location, value)

    def __str__ (self):
        return six.u('Cannot instantiate abstract element %s directly') % (self.element.name(),)

@six.python_2_unicode_compatible
class ContentInNilInstanceError (ElementValidationError):
    """Raised when an element that is marked to be nil is assigned content."""

    instance = None
    """The binding instance which is xsi:nil"""

    content = None
    """The content that was to be assigned to the instance."""

    def __init__ (self, instance, content, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param content: the value for the L{content} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{instance} if possible."""

        self.instance = instance
        self.content = content
        if location is None:
            location = self.instance._location()
        self.location = location
        super(ContentInNilInstanceError, self).__init__(instance, content, location)

    def __str__ (self):
        from pyxb.namespace.builtin import XMLSchema_instance as XSI
        return six.u('%s with %s=true cannot have content') % (self.instance._diagnosticName(), XSI.nil)

class NoNillableSupportError (ElementValidationError):
    """Raised when invoking L{_setIsNil<pyxb.binding.basis._TypeBinding_mixin._setIsNil>} on a type that does not support nillable."""

    instance = None
    """The binding instance on which an inappropriate operation was invoked."""

    def __init__ (self, instance, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{instance} if possible."""
        self.instance = instance
        if location is None:
            location = self.instance._location()
        self.location = location
        super(NoNillableSupportError, self).__init__(instance, location)

@six.python_2_unicode_compatible
class ElementChangeError (ElementValidationError):
    """Attempt to change an element that has a fixed value constraint."""

    element = None
    """The L{pyxb.binding.basis.element} that has a fixed value."""

    value = None
    """The value that was to be assigned to the element."""

    def __init__ (self, element, value, location=None):
        """@param element: the value for the L{element} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{value} if possible."""

        import pyxb.utils.utility
        self.element = element
        self.value = value
        if (location is None) and isinstance(value, pyxb.utils.utility.Locatable_mixin):
            location = value._location()
        self.location = location
        super(ElementChangeError, self).__init__(element, value, location)

    def __str__ (self):
        return six.u('Value %s for element %s incompatible with fixed content') % (self.value, self.element.name())

class ComplexTypeValidationError (ValidationError):
    """Raised when a validation requirement for a complex type is not satisfied."""
    pass

@six.python_2_unicode_compatible
class AbstractInstantiationError (ComplexTypeValidationError):
    """Attempt to create an instance of an abstract complex type.

    These types are analogous to abstract base classes, and cannot be
    created directly.  A type should be used that extends the abstract
    class.

    When an incoming document is missing the xsi:type attribute which
    redirects an element with an abstract type to the correct type,
    the L{node} attribute is provided so the user can get a clue as to
    where the problem occured.  When this exception is a result of
    constructor mis-use in Python code, the traceback will tell you
    where the problem lies.
    """

    type = None
    """The abstract L{pyxb.binding.basis.complexTypeDefinition} subclass used."""

    node = None
    """The L{xml.dom.Element} from which instantiation was attempted, if available."""

    def __init__ (self, type, location, node):
        """@param type: the value for the L{type} attribute.
        @param location: the value for the L{location} attribute.
        @param node: the value for the L{node} attribute."""
        self.type = type
        self.location = location
        self.node = node
        super(AbstractInstantiationError, self).__init__(type, location, node)

    def __str__ (self):
        # If the type is abstract, it has to have a name.
        return six.u('Cannot instantiate abstract type %s directly') % (self.type._ExpandedName,)

@six.python_2_unicode_compatible
class AttributeOnSimpleTypeError (ComplexTypeValidationError):
    """Attempt made to set an attribute on an element with simple type.

    Note that elements with complex type and simple content may have
    attributes; elements with simple type must not."""

    instance = None
    """The simple type binding instance on which no attributes exist."""

    tag = None
    """The name of the proposed attribute."""

    value = None
    """The value proposed to be assigned to the non-existent attribute."""

    def __init__ (self, instance, tag, value, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param tag: the value for the L{tag} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{instance} if possible."""

        self.instance = instance
        self.tag = tag
        self.value = value
        if location is None:
            location = self.instance._location()
        self.location = location
        super(AttributeOnSimpleTypeError, self).__init__(instance, tag, value, location)

    def __str__ (self):
        return six.u('Simple type %s cannot support attribute %s') % (self.instance._Name(), self.tag)

class ContentValidationError (ComplexTypeValidationError):
    """Violation of a complex type content model."""
    pass

@six.python_2_unicode_compatible
class ContentNondeterminismExceededError (ContentValidationError):
    """Content validation exceeded the allowed limits of nondeterminism."""

    instance = None
    """The binding instance being validated."""

    def __init__ (self, instance):
        """@param instance: the value for the L{instance} attribute."""
        self.instance = instance
        super(ContentNondeterminismExceededError, self).__init__(instance)

    def __str__ (self):
        return six.u('Nondeterminism exceeded validating %s') % (self.instance._Name(),)

@six.python_2_unicode_compatible
class SimpleContentAbsentError (ContentValidationError):
    """An instance with simple content was not provided with a value."""

    instance = None
    """The binding instance for which simple content is missing."""

    def __init__ (self, instance, location):
        """@param instance: the value for the L{instance} attribute.
        @param location: the value for the L{location} attribute."""
        self.instance = instance
        self.location = location
        super(SimpleContentAbsentError, self).__init__(instance, location)

    def __str__ (self):
        return six.u('Type %s requires content') % (self.instance._Name(),)

@six.python_2_unicode_compatible
class ExtraSimpleContentError (ContentValidationError):
    """A complex type with simple content was provided too much content."""

    instance = None
    """The binding instance that already has simple content assigned."""

    value = None
    """The proposed addition to that simple content."""

    def __init__ (self, instance, value, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute."""
        self.instance = instance
        self.value = value
        self.location = location
        super(ExtraSimpleContentError, self).__init__(instance, value, location)

    def __str__ (self):
        return six.u('Instance of %s already has simple content value assigned') % (self.instance._Name(),)

@six.python_2_unicode_compatible
class NonPluralAppendError (ContentValidationError):
    """Attempt to append to an element which does not accept multiple instances."""

    instance = None
    """The binding instance containing the element"""

    element_declaration = None
    """The L{pyxb.binding.content.ElementDeclaration} contained in C{instance} that does not accept multiple instances"""

    value = None
    """The proposed addition to the element in the instance"""

    def __init__ (self, instance, element_declaration, value):
        """@param instance: the value for the L{instance} attribute.
        @param element_declaration: the value for the L{element_declaration} attribute.
        @param value: the value for the L{value} attribute."""
        self.instance = instance
        self.element_declaration = element_declaration
        self.value = value
        super(NonPluralAppendError, self).__init__(instance, element_declaration, value)

    def __str__ (self):
        return six.u('Instance of %s cannot append to element %s') % (self.instance._Name(), self.element_declaration.name())

@six.python_2_unicode_compatible
class MixedContentError (ContentValidationError):
    """Non-element content added to a complex type instance that does not support mixed content."""

    instance = None
    """The binding instance."""

    value = None
    """The non-element content."""

    def __init__ (self, instance, value, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute."""
        self.instance = instance
        self.value = value
        self.location = location
        super(MixedContentError, self).__init__(instance, value, location)

    def __str__ (self):
        if self.location is not None:
            return six.u('Invalid non-element content at %s') % (self.location,)
        return six.u('Invalid non-element content')

@six.python_2_unicode_compatible
class UnprocessedKeywordContentError (ContentValidationError):
    """A complex type constructor was provided with keywords that could not be recognized."""

    instance = None
    """The binding instance being constructed."""

    keywords = None
    """The keywords that could not be recognized.  These may have been
    intended to be attributes or elements, but cannot be identified as
    either."""

    def __init__ (self, instance, keywords, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param keywords: the value for the L{keywords} attribute.
        @param location: the value for the L{location} attribute."""
        self.instance = instance
        self.keywords = keywords
        self.location = location
        super(UnprocessedKeywordContentError, self).__init__(instance, keywords, location)

    def __str__ (self):
        return six.u('Unprocessed keywords instantiating %s: %s') % (self.instance._Name(), ' '.join(six.iterkeys(self.keywords)))

class IncrementalElementContentError (ContentValidationError):
    """Element or element-like content could not be validly associated with an sub-element in the content model.

    This exception occurs when content is added to an element during
    incremental validation, such as when positional arguments are used
    in a constructor or material is appended either explicitly or
    through parsing a DOM instance."""

    instance = None
    """The binding for which the L{value} could not be associated with an element."""

    automaton_configuration = None
    """The L{pyxb.binding.content.AutomatonConfiguration} representing the current state of the L{instance} content."""

    value = None
    """The value that could not be associated with allowable content."""

    def __init__ (self, instance, automaton_configuration, value, location=None):
        """@param instance: the value for the L{instance} attribute.
        @param automaton_configuration: the value for the L{automaton_configuration} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute."""
        self.instance = instance
        self.automaton_configuration = automaton_configuration
        self.value = value
        self.location = location
        super(IncrementalElementContentError, self).__init__(instance, automaton_configuration, value, location)

    def _valueDescription (self):
        import xml.dom
        if isinstance(self.value, pyxb.binding.basis._TypeBinding_mixin):
            return self.value._diagnosticName()
        if isinstance(self.value, xml.dom.Node):
            return self.value.nodeName
        return six.text_type(self.value)

@six.python_2_unicode_compatible
class UnrecognizedContentError (IncrementalElementContentError):
    """Element or element-like content could not be validly associated with an sub-element in the content model.

    This exception occurs when content is added to an element during incremental validation."""

    def __str__ (self):
        value = self._valueDescription()
        acceptable = self.automaton_configuration.acceptableContent()
        if 0 == acceptable:
            expect = 'no more content'
        else:
            import pyxb.binding.content
            seen = set()
            names = []
            for u in acceptable:
                if isinstance(u, pyxb.binding.content.ElementUse):
                    n = six.text_type(u.elementBinding().name())
                else:
                    assert isinstance(u, pyxb.binding.content.WildcardUse)
                    n = 'xs:any'
                if not (n in seen):
                    names.append(n)
                    seen.add(n)
            expect = ' or '.join(names)
        location = ''
        if self.location is not None:
            location = ' at %s' % (self.location,)
        return six.u('Invalid content %s%s (expect %s)') % (value, location, expect)

    def details (self):
        import pyxb.binding.basis
        import pyxb.binding.content
        i = self.instance
        rv = [ ]
        if i._element() is not None:
            rv.append('The containing element %s is defined at %s.' % (i._element().name(), i._element().xsdLocation()))
        rv.append('The containing element type %s is defined at %s' % (self.instance._Name(), six.text_type(self.instance._XSDLocation)))
        if self.location is not None:
            rv.append('The unrecognized content %s begins at %s' % (self._valueDescription(), self.location))
        else:
            rv.append('The unrecognized content is %s' % (self._valueDescription(),))
        rv.append('The %s automaton %s in an accepting state.' % (self.instance._Name(), self.automaton_configuration.isAccepting() and "is" or "is not"))
        if isinstance(self.instance, pyxb.binding.basis.complexTypeDefinition) and self.instance._IsMixed():
            rv.append('Character information content would be permitted.')
        acceptable = self.automaton_configuration.acceptableContent()
        if 0 == len(acceptable):
            rv.append('No elements or wildcards would be accepted at this point.')
        else:
            rv.append('The following element and wildcard content would be accepted:')
            rv2 = []
            for u in acceptable:
                if isinstance(u, pyxb.binding.content.ElementUse):
                    rv2.append('An element %s per %s' % (u.elementBinding().name(), u.xsdLocation()))
                else:
                    assert isinstance(u, pyxb.binding.content.WildcardUse)
                    rv2.append('A wildcard per %s' % (u.xsdLocation(),))
            rv.append('\t' + '\n\t'.join(rv2))
        return '\n'.join(rv)

class BatchElementContentError (ContentValidationError):
    """Element/wildcard content cannot be reconciled with the required content model.

    This exception occurs in post-construction validation using a
    fresh validating automaton."""

    instance = None
    """The binding instance being constructed."""

    fac_configuration = None
    """The L{pyxb.utils.fac.Configuration} representing the current state of the L{instance} automaton."""

    symbols = None
    """The sequence of symbols that were accepted as content prior to the error."""

    symbol_set = None
    """The leftovers from L{pyxb.binding.basis.complexTypeDefinition._symbolSet} that could not be reconciled with the content model."""

    def __init__ (self, instance, fac_configuration, symbols, symbol_set):
        """@param instance: the value for the L{instance} attribute.
        @param fac_configuration: the value for the L{fac_configuration} attribute.
        @param symbols: the value for the L{symbols} attribute.
        @param symbol_set: the value for the L{symbol_set} attribute."""
        self.instance = instance
        self.fac_configuration = fac_configuration
        self.symbols = symbols
        self.symbol_set = symbol_set
        super(BatchElementContentError, self).__init__(instance, fac_configuration, symbols, symbol_set)

    def details (self):
        import pyxb.binding.basis
        import pyxb.binding.content
        i = self.instance
        rv = [ ]
        if i._element() is not None:
            rv.append('The containing element %s is defined at %s.' % (i._element().name(), i._element().xsdLocation()))
        rv.append('The containing element type %s is defined at %s' % (self.instance._Name(), six.text_type(self.instance._XSDLocation)))
        rv.append('The %s automaton %s in an accepting state.' % (self.instance._Name(), self.fac_configuration.isAccepting() and "is" or "is not"))
        if self.symbols is None:
            rv.append('Any accepted content has been stored in instance')
        elif 0 == len(self.symbols):
            rv.append('No content has been accepted')
        else:
            rv.append('The last accepted content was %s' % (self.symbols[-1].value._diagnosticName(),))
        if isinstance(self.instance, pyxb.binding.basis.complexTypeDefinition) and self.instance._IsMixed():
            rv.append('Character information content would be permitted.')
        acceptable = self.fac_configuration.acceptableSymbols()
        if 0 == len(acceptable):
            rv.append('No elements or wildcards would be accepted at this point.')
        else:
            rv.append('The following element and wildcard content would be accepted:')
            rv2 = []
            for u in acceptable:
                if isinstance(u, pyxb.binding.content.ElementUse):
                    rv2.append('An element %s per %s' % (u.elementBinding().name(), u.xsdLocation()))
                else:
                    assert isinstance(u, pyxb.binding.content.WildcardUse)
                    rv2.append('A wildcard per %s' % (u.xsdLocation(),))
            rv.append('\t' + '\n\t'.join(rv2))
        if (self.symbol_set is None) or (0 == len(self.symbol_set)):
            rv.append('No content remains unconsumed')
        else:
            rv.append('The following content was not processed by the automaton:')
            rv2 = []
            for (ed, syms) in six.iteritems(self.symbol_set):
                if ed is None:
                    rv2.append('xs:any (%u instances)' % (len(syms),))
                else:
                    rv2.append('%s (%u instances)' % (ed.name(), len(syms)))
            rv.append('\t' + '\n\t'.join(rv2))
        return '\n'.join(rv)

class IncompleteElementContentError (BatchElementContentError):
    """Validation of an instance failed to produce an accepting state.

    This exception occurs in batch-mode validation."""
    pass

class UnprocessedElementContentError (BatchElementContentError):
    """Validation of an instance produced an accepting state but left element material unconsumed.

    This exception occurs in batch-mode validation."""
    pass

class InvalidPreferredElementContentError (BatchElementContentError):
    """Use of a preferred element led to inability to generate a valid document"""

    preferred_symbol = None
    """The element symbol which was not accepted."""

    def __init__ (self, instance, fac_configuration, symbols, symbol_set, preferred_symbol):
        """@param instance: the value for the L{instance} attribute.
        @param fac_configuration: the value for the L{fac_configuration} attribute.
        @param symbols: the value for the L{symbols} attribute.
        @param symbol_set: the value for the L{symbol_set} attribute.
        @param preferred_symbol: the value for the L{preferred_symbol} attribute.
        """
        self.instance = instance
        self.fac_configuration = fac_configuration
        self.symbols = symbols
        self.symbol_set = symbol_set
        self.preferred_symbol = preferred_symbol
        # Bypass immediate parent so we preserve the last argument
        super(BatchElementContentError, self).__init__(instance, fac_configuration, symbols, symbol_set, preferred_symbol)

@six.python_2_unicode_compatible
class OrphanElementContentError (ContentValidationError):
    """An element expected to be used in content is not present in the instance.

    This exception occurs in batch-mode validation when
    L{pyxb.ValidationConfig.contentInfluencesGeneration} applies,
    L{pyxb.ValidationConfig.orphanElementInContent} is set to
    L{pyxb.ValidationConfig.RAISE_EXCEPTION}, and the content list
    includes an element that is not in the binding instance
    content.
    """

    instance = None
    """The binding instance."""

    preferred = None
    """An element value from the L{instance} L{content<pyxb.binding.basis.complexTypeDefinition.content>} list which was not found in the L{instance}."""

    def __init__ (self, instance, preferred):
        """@param instance: the value for the L{instance} attribute.
        @param preferred: the value for the L{preferred} attribute.
        """
        self.instance = instance
        self.preferred = preferred
        super(OrphanElementContentError, self).__init__(instance, preferred)

    def __str__ (self):
        return six.u('Preferred content element not found in instance')

@six.python_2_unicode_compatible
class SimpleTypeValueError (ValidationError):
    """Raised when a simple type value does not satisfy its constraints."""
    type = None
    """The L{pyxb.binding.basis.simpleTypeDefinition} that constrains values."""

    value = None
    """The value that violates the constraints of L{type}.  In some
    cases this is a tuple of arguments passed to a constructor that
    failed with a built-in exception likeC{ValueError} or
    C{OverflowError}."""

    def __init__ (self, type, value, location=None):
        """@param type: the value for the L{type} attribute.
        @param value: the value for the L{value} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{value} if possible."""
        import pyxb.utils.utility
        self.type = type
        self.value = value
        if (location is None) and isinstance(value, pyxb.utils.utility.Locatable_mixin):
            location = value._location()
        self.location = location
        super(SimpleTypeValueError, self).__init__(type, value, location)

    def __str__ (self):
        import pyxb.binding.basis
        if isinstance(self.value, pyxb.binding.basis._TypeBinding_mixin):
            return six.u('Type %s cannot be created from %s: %s') % (self.type._Name(), self.value._Name(), self.value)
        return six.u('Type %s cannot be created from: %s') % (self.type._Name(), self.value)

@six.python_2_unicode_compatible
class SimpleListValueError (SimpleTypeValueError):
    """Raised when a list simple type contains a member that does not satisfy its constraints.

    In this case, L{type} is the type of the list, and value
    C{type._ItemType} is the type for which the L{value} is
    unacceptable."""

    def __str__ (self):
        return six.u('Member type %s of list type %s cannot accept %s') % (self.type._ItemType._Name(), self.type._Name(), self.value)

@six.python_2_unicode_compatible
class SimpleUnionValueError (SimpleTypeValueError):
    """Raised when a union simple type contains a member that does not satisfy its constraints.

    In this case, L{type} is the type of the union, and the value
    C{type._MemberTypes} is the set of types for which the value is
    unacceptable.

    The L{value} itself is the tuple of arguments passed to the
    constructor for the union."""

    def __str__ (self):
        return six.u('No memberType of %s can be constructed from %s') % (self.type._Name(), self.value)

@six.python_2_unicode_compatible
class SimpleFacetValueError (SimpleTypeValueError):
    """Raised when a simple type value does not satisfy a facet constraint.

    This extends L{SimpleTypeValueError} with the L{facet} field which
    can be used to determine why the value is unacceptable."""

    type = None
    """The L{pyxb.binding.basis.simpleTypeDefinition} that constrains values."""

    value = None
    """The value that violates the constraints of L{type}.  In some
    cases this is a tuple of arguments passed to a constructor that
    failed with a built-in exception likeC{ValueError} or
    C{OverflowError}."""

    facet = None
    """The specific facet that is violated by the value."""

    def __init__ (self, type, value, facet, location=None):
        """@param type: the value for the L{type} attribute.
        @param value: the value for the L{value} attribute.
        @param facet: the value for the L{facet} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{value} if possible."""
        import pyxb.utils.utility

        self.type = type
        self.value = value
        self.facet = facet
        if (location is None) and isinstance(value, pyxb.utils.utility.Locatable_mixin):
            location = value._location()
        self.location = location
        # Bypass immediate parent
        super(SimpleTypeValueError, self).__init__(type, value, facet)

    def __str__ (self):
        return six.u('Type %s %s constraint violated by value %s') % (self.type._Name(), self.facet._Name, self.value)

class SimplePluralValueError (SimpleTypeValueError):
    """Raised when context requires a plural value.

    Unlike L{SimpleListValueError}, in this case the plurality is
    external to C{type}, for example when an element has simple
    content and allows multiple occurrences."""
    pass

class AttributeValidationError (ValidationError):
    """Raised when an attribute requirement is not satisfied."""

    type = None
    """The L{pyxb.binding.basis.complexTypeDefinition} subclass of the instance."""

    tag = None
    """The name of the attribute."""

    instance = None
    """The binding instance, if available."""

    def __init__ (self, type, tag, instance=None, location=None):
        """@param type: the value for the L{type} attribute.
        @param tag: the value for the L{tag} attribute.
        @param instance: the value for the L{instance} attribute.
        @param location: the value for the L{location} attribute.  Default taken from C{instance} if possible.
        """
        import pyxb.utils.utility as utility
        self.type = type
        self.tag = tag
        self.instance = instance
        if (location is None) and isinstance(instance, utility.Locatable_mixin):
            location = instance._location()
        self.location = location
        super(AttributeValidationError, self).__init__(type, tag, instance, location)

@six.python_2_unicode_compatible
class UnrecognizedAttributeError (AttributeValidationError):
    """Attempt to reference an attribute not sanctioned by content model."""
    def __str__ (self):
        return six.u('Attempt to reference unrecognized attribute %s in type %s') % (self.tag, self.type)

@six.python_2_unicode_compatible
class ProhibitedAttributeError (AttributeValidationError):
    """Raised when an attribute that is prohibited is set or referenced in an element."""
    def __str__ (self):
        return six.u('Attempt to reference prohibited attribute %s in type %s') % (self.tag, self.type)

@six.python_2_unicode_compatible
class MissingAttributeError (AttributeValidationError):
    """Raised when an attribute that is required is missing in an element."""
    def __str__ (self):
        return six.u('Instance of %s lacks required attribute %s') % (self.type, self.tag)

@six.python_2_unicode_compatible
class AttributeChangeError (AttributeValidationError):
    """Attempt to change an attribute that has a fixed value constraint."""
    def __str__ (self):
        return six.u('Cannot change fixed attribute %s in type %s') % (self.tag, self.type)

class BindingError (PyXBException):
    """Raised when the bindings are mis-used.

    These are not validation errors, but rather structural errors.
    For example, attempts to extract complex content from a type that
    requires simple content, or vice versa.  """

@six.python_2_unicode_compatible
class NotSimpleContentError (BindingError):
    """An operation that requires simple content was invoked on a
    complex type instance that does not have simple content."""

    instance = None
    """The binding instance which should have had simple content."""

    def __init__ (self, instance):
        """@param instance: the binding instance that was mis-used.
        This will be available in the L{instance} attribute."""
        self.instance = instance
        super(BindingError, self).__init__(instance)
    pass

    def __str__ (self):
        return six.u('type %s does not have simple content') % (self.instance._Name(),)

@six.python_2_unicode_compatible
class NotComplexContentError (BindingError):
    """An operation that requires a content model was invoked on a
    complex type instance that has empty or simple content."""

    instance = None
    """The binding instance which should have had a content model."""

    def __init__ (self, instance):
        """@param instance: the binding instance that was mis-used.
        This will be available in the L{instance} attribute."""
        self.instance = instance
        super(BindingError, self).__init__(instance)

    def __str__ (self):
        return six.u('type %s has simple/empty content') % (self.instance._Name(),)

@six.python_2_unicode_compatible
class ReservedNameError (BindingError):
    """Reserved name set in binding instance."""

    instance = None
    """The binding instance."""

    name = None
    """The name that was caught being assigned"""

    def __init__ (self, instance, name):
        """@param instance: the value for the L{instance} attribute.
        p@param name: the value for the L{name} attribute."""
        self.instance = instance
        self.name = name
        super(ReservedNameError, self).__init__(instance, name)

    def __str__ (self):
        return six.u('%s is a reserved name within %s') % (self.name, self.instance._Name())

class PyXBError (Exception):
    """Base class for exceptions that indicate a problem that the user probably can't fix."""
    pass

class UsageError (PyXBError):
    """Raised when the code detects user violation of an API."""

class LogicError (PyXBError):
    """Raised when the code detects an implementation problem."""

class IncompleteImplementationError (LogicError):
    """Raised when required capability has not been implemented.

    This is only used where it is reasonable to expect the capability
    to be present, such as a feature of XML schema that is not
    supported (e.g., the redefine directive)."""



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
from pyxb.utils.templates import *

_dictionary = { 'one' : 'un'
              , 'two' : 'dau'
              , 'three' : 'tri'
              , 'un' : 1
              , 'dau': 2
              , 'tri': 3
              , 'empty': ''
              , 'defined': 'value'
              }

class IfDefinedPatternTestCase (unittest.TestCase):
    dictionary = _dictionary

    def testNoSubst (self):
        self.assertEqual('un', replaceInText('%{?one?}', **self.dictionary))
        self.assertEqual('', replaceInText('%{?four?}', **self.dictionary))

    def testPlainSubst (self):
        self.assertEqual('one is defined', replaceInText('%{?one?+one is defined?}', **self.dictionary))
        self.assertEqual('', replaceInText('%{?four?+four is defined?}', **self.dictionary))
        self.assertEqual('four is not defined', replaceInText('%{?four?+@? is defined?-?@ is not defined?}', **self.dictionary))
        self.assertEqual('name', replaceInText('%{?context?+%{?@}.?}name'))
        self.assertEqual('name', replaceInText('%{?context?+%{?@}.?}name', context=None))
        self.assertEqual('owner.name', replaceInText('%{?context?+%{?@}.?}name', context='owner'))

    def testWithSubst (self):
        self.assertEqual('un and un are two', replaceInText('%{?one?+%{?@} and %{?@} are two?-what is ?@??}', **self.dictionary))
        self.assertEqual('what is "four"?', replaceInText('%{?four?+%{?@} and %{?@} are two?-what is "?@"??}', **self.dictionary))


class ConditionalPatternTestCase (unittest.TestCase):
    dictionary = _dictionary

    def testBasic (self):
        self.assertEqual('three is defined', replaceInText('%{?three??three is defined?:three is not defined?}', **self.dictionary))
        self.assertEqual("%{EXCEPTION: name 'four' is not defined}", replaceInText('%{?four??four is defined?:four is not defined?}', **self.dictionary))
        self.assertEqual('value', replaceInText('%{?defined??%{defined}?:pass?}', **self.dictionary))
        self.assertEqual('pass', replaceInText('%{?empty??%{empty}?:pass?}', **self.dictionary))

    def testHalfExpressions (self):
        self.assertEqual('value is three', replaceInText('%{?3 == un+dau??value is three?}', **self.dictionary))
        self.assertEqual('', replaceInText('%{?3 == un-dau??value is three?}', **self.dictionary))
        self.assertEqual('good 1 == un', replaceInText('%{?1 == un??good ?@?:bad ?@?}', **self.dictionary))
        self.assertEqual('bad 2 == un', replaceInText('%{?2 == un??good ?@?:bad ?@?}', **self.dictionary))
        self.assertEqual('''
        if runtime_test:
            print 'Good on 1 == un'
''', replaceInText('''
        %{?1 == un??if runtime_test:
            print 'Good on ?@'
?}''', **self.dictionary))

    def testExpressions (self):
        self.assertEqual('value is three', replaceInText('%{?3 == un+dau??value is three?:value is not three?}', **self.dictionary))
        self.assertEqual('value is not three', replaceInText('%{?3 == un-dau??value is three?:value is not three?}', **self.dictionary))

    def testNesting (self):
        self.assertEqual('tri', replaceInText('%{?3 == un+dau??%{three}?:not %{three}?}', **self.dictionary))

class IdPatternTestCase (unittest.TestCase):
    dictionary = _dictionary

    def testNoChange (self):
        cases = [ 'plain text'
                , '   leading whitespace'
                , 'trailing whitespace '
                , '''Multiline
text''' ] # '''
        for c in cases:
            self.assertEqual(c, replaceInText(c, **{}))

    def testSimpleSubstitution (self):
        self.assertEqual('un', replaceInText('%{one}', **self.dictionary))
        self.assertEqual('un and dau', replaceInText('%{one} and %{two}', **self.dictionary))
        self.assertEqual('''Line un
Line dau
Line tri
''', replaceInText('''Line %{one}
Line %{two}
Line %{three}
''', **self.dictionary))

    def testMissing (self):
        self.assertEqual('%{MISSING:four}', replaceInText('%{four}', **self.dictionary))

if __name__ == '__main__':
    #print replaceInText('%{?three%?three is defined%:three is not defined?}', ConditionalPatternTestCase.dictionary)
    #print "\n".join(_substConditionalPattern.match('%{?three%?three is defined%:three is not defined?}').groups())
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node
import sys

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-mg-all.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils
def ToDOM (instance):
    return instance.toDOM().documentElement

import unittest

class TestMGAll (unittest.TestCase):
    def testRequired (self):
        xmlt = '<ns1:required xmlns:ns1="URN:test-mg-all"/>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, required.createFromDOM, dom.documentElement)

        xmlt = '<ns1:required xmlns:ns1="URN:test-mg-all"><first/><second/><third/></ns1:required>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = required.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, required.memberElement('first').typeDefinition()))
        self.assertTrue(isinstance(instance.second, required.memberElement('second').typeDefinition()))
        self.assertTrue(isinstance(instance.third, required.memberElement('third').typeDefinition()))

    def testRequiredMisordered (self):
        xmlt = '<ns1:required xmlns:ns1="URN:test-mg-all"><third/><first/><second/></ns1:required>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = required.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, required.memberElement('first').typeDefinition()))
        self.assertTrue(isinstance(instance.second, required.memberElement('second').typeDefinition()))
        self.assertTrue(isinstance(instance.third, required.memberElement('third').typeDefinition()))

    def testRequiredTooMany (self):
        xmlt = '<ns1:required xmlns:ns1="URN:test-mg-all"><third/><first/><second/><third/></ns1:required>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, required.createFromDOM, dom.documentElement)

    def testThirdOptional (self):
        xmlt = '<ns1:thirdOptional xmlns:ns1="URN:test-mg-all"><first/><second/></ns1:thirdOptional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = thirdOptional.Factory(_dom_node=dom.documentElement)
        self.assertTrue(isinstance(instance.first, thirdOptional._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.second, thirdOptional._ElementMap['second'].elementBinding().typeDefinition()))
        self.assertTrue(instance.third is None)

        xmlt = '<ns1:thirdOptional xmlns:ns1="URN:test-mg-all"><first/><second/><third/></ns1:thirdOptional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = thirdOptional.Factory(_dom_node=dom.documentElement)
        self.assertTrue(isinstance(instance.first, thirdOptional._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.second, thirdOptional._ElementMap['second'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.third, thirdOptional._ElementMap['third'].elementBinding().typeDefinition()))

        xmlt = '<ns1:thirdOptional xmlns:ns1="URN:test-mg-all"><first/><second/><third/><first/></ns1:thirdOptional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, thirdOptional.Factory, _dom_node=dom.documentElement)

    def testOptional (self):
        xmlt = '<ns1:optional xmlns:ns1="URN:test-mg-all"/>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = optional.createFromDOM(dom.documentElement)
        self.assertTrue(instance.first is None)
        self.assertTrue(instance.second is None)
        self.assertTrue(instance.third is None)

        xmlt = '<ns1:optional xmlns:ns1="URN:test-mg-all"><first/><second/><third/></ns1:optional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = optional.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, optional.memberElement('first').typeDefinition()))
        self.assertTrue(isinstance(instance.second, optional.memberElement('second').typeDefinition()))
        self.assertTrue(isinstance(instance.third, optional.memberElement('third').typeDefinition()))

        xmlt = '<ns1:optional xmlns:ns1="URN:test-mg-all"><first/><third/></ns1:optional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = optional.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, optional.memberElement('first').typeDefinition()))
        self.assertTrue(instance.second is None)
        self.assertTrue(isinstance(instance.third, optional.memberElement('third').typeDefinition()))

        xmlt = '<ns1:optional xmlns:ns1="URN:test-mg-all"><third/></ns1:optional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = optional.createFromDOM(dom.documentElement)
        self.assertTrue(instance.first is None)
        self.assertTrue(instance.second is None)
        self.assertTrue(isinstance(instance.third, optional.memberElement('third').typeDefinition()))

    def testOptionalTooMany (self):
        xmlt = '<ns1:optional xmlns:ns1="URN:test-mg-all"><third/><first/><third/></ns1:optional>'
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, optional.createFromDOM, dom.documentElement)

    def stripMembers (self, xmlt, body):
        for b in body:
            xmlt = xmlt.replace('<%s/>' % (b,), 'X')
        return xmlt

    def testMany (self):
        for body in [ "abcdefgh", "fghbcd", "bfgcahd" ]:
            xmlt = '<ns1:many xmlns:ns1="URN:test-mg-all">%s</ns1:many>' % (''.join([ '<%s/>' % (_x,) for _x in body ]),)
            dom = pyxb.utils.domutils.StringToDOM(xmlt)
            instance = many.createFromDOM(dom.documentElement)
            instance.validateBinding()
            xml2d = ToDOM(instance).toxml("utf-8")
            xml2t = xml2d.decode('utf-8')
            rev = self.stripMembers(xml2t, body)
            self.assertEqual('<ns1:many xmlns:ns1="URN:test-mg-all">%s</ns1:many>' % (''.join(len(body)*['X']),), rev)
        many_a = many.memberElement('a')
        many_c = many.memberElement('c')
        many_d = many.memberElement('d')
        many_e = many.memberElement('e')
        many_f = many.memberElement('f')
        many_g = many.memberElement('g')
        many_h = many.memberElement('h')
        instance = many(a=many_a(), c=many_c(), d=many_d(), e=many_e(), f=many_f(), g=many_g(), h=many_h())
        self.assertRaises(pyxb.IncompleteElementContentError, instance.validateBinding)
        self.assertRaises(pyxb.IncompleteElementContentError, ToDOM, instance)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(IncompleteElementContentError) as cm:
                instance.validateBinding()
            acceptable = cm.exception.fac_configuration.acceptableSymbols()
            self.assertEqual(1, len(acceptable))
            self.assertEqual(many.memberElement('b'), acceptable[0].elementDeclaration().elementBinding())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified">

    <xs:simpleType name="TestEnum">
        <xs:restriction base="xs:string">
            <xs:enumeration value="foo"/>
            <xs:enumeration value="bar"/>
        </xs:restriction>
    </xs:simpleType>

    <xs:element name="root" type="TestEnum" nillable="true"/>

</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0195 (unittest.TestCase):

    def testValidNil (self):
        xmls = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="file:test_enum.xsd" xsi:nil="true"/>'
        instance = CreateFromDocument(xmls)
        self.assertTrue(instance._isNil())

    def testNotNil (self):
        xmls = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="file:test_enum.xsd" xsi:nil="true">foo</root>'
        self.assertRaises(pyxb.ContentInNilInstanceError, CreateFromDocument, xmls)

    def testInvalidNil (self):
        xmls = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="file:test_enum.xsd"/>'
        self.assertRaises(pyxb.SimpleFacetValueError, CreateFromDocument, xmls)

    def testValidNotNil (self):
        xmls = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="file:test_enum.xsd">foo</root>'
        instance = CreateFromDocument(xmls)
        self.assertFalse(instance._isNil())
        self.assertEqual('foo', instance)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_NMTOKENS (unittest.TestCase):
    def testBasicLists (self):
        v = xsd.NMTOKENS([ "one", "_two", "three" ])
        self.assertEqual(3, len(v))
        self.assertTrue(isinstance(v[0], xsd.NMTOKEN))
        self.assertEqual("one", v[0])

    def testStringLists (self):
        v = xsd.NMTOKENS("one _two three")
        self.assertEqual(3, len(v))
        self.assertEqual("one", v[0])
        self.assertTrue(isinstance(v[0], xsd.NMTOKEN))
        self.assertRaises(SimpleTypeValueError, xsd.NMTOKENS, 'string with b@d id')

    def testInsertion (self):
        tup = ('a', 'b', 'c')
        v = xsd.NMTOKENS(tup)
        self.assertEqual(tup, tuple(v))
        v[1] = 'B'
        self.assertEqual(('a', 'B', 'c'), tuple(v))
        v[1:2] = ['B', 'b']
        self.assertEqual(('a', 'B', 'b', 'c'), tuple(v))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
import pyxb.utils.utility
from pyxb.utils.utility import MakeIdentifier

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
   <xs:complexType name="simple_type">
      <xs:simpleContent>
         <xs:extension base="xs:string"/>
      </xs:simpleContent>
      <!-- attribute cannot be here, must be in xs:extension -->
      <xs:attribute name="is_clean" type="xs:boolean"/>
   </xs:complexType>
   <xs:element name="simple_element" type="simple_type"/>
</xs:schema>'''

import unittest

class TestTrac0148 (unittest.TestCase):
    def testProcessing (self):
        self.assertRaises(pyxb.SchemaValidationError, pyxb.binding.generate.GeneratePython, schema_text=xsd)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import base
import profile
import unittest

class TestTrac0073 (unittest.TestCase):
    def testProfile (self):
        amap = profile.AbstractFeatureBaseType._AttributeMap
        self.assertEqual(1, len(amap))
        base_id = base.Namespace.createExpandedName('id')
        self.assertTrue(base_id in amap)

if __name__ == '__main__':
    unittest.main()
